/******************************************************************************
$Header: /Repository/drv/hpi/hpi_internal.h,v 1.129 2011/02/03 14:26:11 as-age Exp $

HPI internal definitions

(C) Copyright AudioScience Inc. 1996-2009
******************************************************************************/

#ifndef _HPI_INTERNAL_H_
#define _HPI_INTERNAL_H_

#include "hpi.h"
/** maximum number of memory regions mapped to an adapter */
#define HPI_MAX_ADAPTER_MEM_SPACES (2)

/* Each OS needs its own hpios.h */
#include <hpios.h>

/* SANITISE DELETE */
#if 1

#ifdef HPI_64BIT
#error Use HPI_BUILD_64BIT
#endif

#ifdef HPI_KERNEL_MODE
#error Use HPI_BUILD_KERNEL_MODE
#endif

#ifdef HPI_DISABLE_MUTEXES
#error Use HPI_BUILD_DISABLE_MUTEXES
#endif

#ifdef HPI_MULTIINTERFACE
#error  Use HPI_BUILD_MULTIINTERFACE
#endif

#ifdef HPI_INCLUDE_INTERNAL
#error Use HPI_BUILD_INCLUDE_INTERNAL
#endif

#if defined HPI_INCLUDE_UDP
#error   HPI_INCLUDE_UDP is obsolete
#endif

#ifdef HPI_DSPCODE_FILE
#error  use HPI_BUILD_DSPCODE_FILE
#endif

#if defined HPI_USEUDPDLL
#error  use HPI_BUILD_USEUDPDLL
#endif

#if defined HPI_SUPPORT_SSX2
#error  Use HPI_BUILD_SSX2
#endif

#ifdef HPI_INCLUDE_HPI6205
#error Use HPI_BUILD_INCLUDE_HPI6205
#endif

#ifdef HPI_INCLUDE_HPI6000
#error Use HPI_BUILD_INCLUDE_HPI6000
#endif

#ifdef WDM_MINIPORT
#error use HPI_BUILD_WDM_MINIPORT
#endif

#ifdef HPI_WDM_MONOLITHIC
#error use HPI_BUILD_WDM_MONOLITHIC
#endif

#ifdef HPI_VISTA_DEVNAMES
#error use HPI_BUILD_VISTA_DEVNAMES
#endif

#ifdef NO_RECORD_MUX
#error use HPI_BUILD_NO_RECORD_MUX
#endif

#ifdef HPI_CACHEBBMSTREAMBUFS
#error use  HPI_BUILD_CACHEBBMSTREAMBUFS
#endif

#ifdef HPI_EXPORTUDPMSGENTRY
#error use HPI_BUILD_EXPORTUDPMSGENTRY
#endif

#endif
/* SANITISE DELETE END */


#if defined(HPI_OS_WDM) || defined(HPI_OS_WIN32_USER)

/* HPI IOCTL definitions, shared by windows user and kernel modes */
#define HPI_IOCTL_WINNT_OLD  CTL_CODE(50000,0xA00,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define HPI_IOCTL_WINNT      CTL_CODE(50000,0xA01,METHOD_BUFFERED,FILE_ANY_ACCESS)
#define HPI_IOCTL_WIN95      0x101

/* HPI start device failure log filename shared by windows user and kernel modes */
#define HPI_ASISDFL_FILENAME	L"asi_failure.txt"

#ifndef HPI_API
#define HPI_API(a) a
#endif

#ifndef HPI_CALLBACK
#define HPI_CALLBACK
#endif

#endif	// not LINUX

#ifndef HPI_OS_LINUX_KERNEL

#ifndef __user
#define __user
#endif
#ifndef __iomem
#define __iomem
#endif

#endif /* HPI_OS_LINUX_KERNEL */

/* SANITISE DELETE */
#ifndef HPIOS_MEMWRITEBLK32
#define HPIOS_MEMWRITEBLK32(to, from, nwords) do {\
		int i;\
		uint32_t *src = from;\
		for (i = 0; i < nwords; i++)\
			HPIOS_MEMWRITE32(to, *src++);\
} while (0)
#endif

#ifndef HPIOS_MEMREADBLK32
#define HPIOS_MEMREADBLK32(from, to, nwords)  do {\
	int i;\
	uint32_t *dst = to;\
	for (i = 0; i < nwords; i++)\
		*dst++ = HPIOS_MEMREAD32(from);\
} while (0)
#endif
/* SANITISE DELETE END */

#ifndef HPI_LOCKING

#define DSPLOCK_TYPE int
#define OSTYPE_VALIDFLAG int

#ifndef HPI_ALIST_LOCKING
struct hpios_spinlock {
	DSPLOCK_TYPE lock;
};
#endif

#define HpiOs_Msgxlock_Init(obj)
#define HpiOs_Msgxlock_Lock(obj)
#define HpiOs_Msgxlock_Unlock(obj)

#define HpiOs_Dsplock_Init(obj)
#define HpiOs_Dsplock_Lock(obj)
#define HpiOs_Dsplock_Unlock(obj)

#endif

#ifndef HPI_ALIST_LOCKING

#define HpiOs_Alistlock_Init(obj)
#define HpiOs_Alistlock_Lock(obj)
#define HpiOs_Alistlock_Unlock(obj)

#endif

#ifndef HPI_OS_LINUX_KERNEL
/* memory allocation */
#ifndef HpiOs_MemAlloc
void *HpiOs_MemAlloc(uint32_t dwSize);
#endif
#ifndef HpiOs_MemAllocZero
void *HpiOs_MemAllocZero(uint32_t dwSize);
#endif
#ifndef HpiOs_MemFree
void HpiOs_MemFree(void *ptr);
#endif
#endif

/* physical memory allocation */
#ifndef HPI_NO_OS_LOCKEDMEM_OPS
void HpiOs_LockedMem_Init(
	void
);
void HpiOs_LockedMem_FreeAll(
	void
);
#ifdef HPI_BUILD_CACHEBBMSTREAMBUFS
hpi_err_t HpiOs_LockedMem_Prepare(
	uint32_t dwMin,
	uint32_t dwMax,
	uint32_t dwSize,
	struct pci_dev *pOsReference
);
hpi_err_t HpiOs_LockedMem_Unprepare(
	struct pci_dev *pOsReference
);
#else
#define HpiOs_LockedMem_Prepare(a, b, c, d);
#define HpiOs_LockedMem_Unprepare(a)
#endif

/** Allocate and map an area of locked memory for bus master DMA operations.

On success, *pLockedMemeHandle is a valid handle, and 0 is returned
On error *pLockedMemHandle marked invalid, non-zero returned.

If this function succeeds, then HpiOs_LockedMem_GetVirtAddr() and
HpiOs_LockedMem_GetPyhsAddr() will always succed on the returned handle.
*/
hpi_err_t HpiOs_LockedMem_Alloc(
	HpiOs_LockedMem_Handle * pLockedMemHandle,  /**< memory handle */
	uint32_t dwSize, /**< Size in bytes to allocate */
	struct pci_dev *pOsReference
	/**< OS specific data required for memory allocation */
#ifdef HPI_LOG_LOCKED_MEM
	, const char *pFileName, int nLineNumber
#endif
);

/** Free mapping and memory represented by LockedMemHandle

Frees any resources, then invalidates the handle.
Returns 0 on success, 1 if handle is invalid.

*/
hpi_err_t HpiOs_LockedMem_Free(
	HpiOs_LockedMem_Handle *LockedMemHandle
#ifdef HPI_LOG_LOCKED_MEM
	, const char *pFileName, int nLineNumber
#endif
);

/** Get the physical PCI address of memory represented by LockedMemHandle.

If handle is invalid *pPhysicalAddr is set to zero and return 1
*/
hpi_err_t HpiOs_LockedMem_GetPhysAddr(
	HpiOs_LockedMem_Handle *LockedMemHandle,
	uint32_t *pPhysicalAddr
);

/** Get the CPU address of of memory represented by LockedMemHandle.

If handle is NULL *ppvVirtualAddr is set to NULL and return 1
*/
hpi_err_t HpiOs_LockedMem_GetVirtAddr(
	HpiOs_LockedMem_Handle *LockedMemHandle,
	void  **ppvVirtualAddr
);

/** Check that handle is valid
i.e it represents a valid memory area
*/
hpi_err_t HpiOs_LockedMem_Valid(
	HpiOs_LockedMem_Handle *LockedMemHandle
);

/* SANITISE DELETE */
// Could this move to hpios_wdm.h?
#ifdef HPI_LOG_LOCKED_MEM
#define HpiOs_LockedMem_Alloc(a,b,c)	\
	HpiOs_LockedMem_Alloc(a,b,c,__FILE__,__LINE__)
#define HpiOs_LockedMem_Free(a)		\
	HpiOs_LockedMem_Free(a,__FILE__,__LINE__)
#define HPI_DEBUG_LEVEL_LOCKED_MEM_DEBUG_LEVEL HPI_DEBUG_LEVEL_WARNING
#define HPI_DEBUG_FLAG_LOCKED_MEM_DEBUG_LEVEL
#else
#define HPI_DEBUG_LEVEL_LOCKED_MEM_DEBUG_LEVEL HPI_DEBUG_LEVEL_DEBUG
#define HPI_DEBUG_FLAG_LOCKED_MEM_DEBUG_LEVEL
#endif
/* SANITISE DELETE END */

#endif

/* timing/delay */
void HpiOs_DelayMicroSeconds(
	uint32_t dwNumMicroSec
);

#ifndef HPI_NO_OS_FILE_OPS
#ifndef HpiOs_fopen_rb
/* functions not implemented as macros in OS.h files */
hpi_err_t HpiOs_fopen_rb(
	const char *filename,
	HpiOs_FILE * pFile,
	uint32_t
	*pdwOsErrorCode
);
int HpiOs_fseek(
	HpiOs_FILE stream,
	long offset,
	int origin
);
int HpiOs_fread(
	void *buffer,
	size_t size,
	size_t count,
	HpiOs_FILE stream
);
int HpiOs_fclose(
	HpiOs_FILE stream
);
#endif

const char *HpiOs_GetDspCodePath(
	uint32_t nAdapter
);
void HpiOs_SetDspCodePath(
	const char *pPath
);
#endif

struct hpi_message;
struct hpi_response;

typedef void HPI_HandlerFunc(
	struct hpi_message *,
	struct hpi_response *
);

#ifndef HPI_OS_LINUX_KERNEL
#define PCI_ANY_ID 0xffff

#define DRIVER_DATA_TYPE	HPI_HandlerFunc *
struct pci_device_id {
	uint16_t vendor, device;
	uint16_t subvendor, subdevice;
	uint16_t pci_class, class_mask;
	DRIVER_DATA_TYPE driver_data;
};

/** A 6 byte ethernet mac address */
typedef uint8_t hpi_ethernet_mac_adr_t[6];
#define HPI_ETHERNET_MAC_ADR hpi_ethernet_mac_adr_t
#endif

/* SANITISE DELETE */
/* A few convenience macros */
#ifndef ARRAY_SIZE
#define ARRAY_SIZE(A) (sizeof(A)/sizeof((A)[0]))
#endif

#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif

#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif

#ifndef __stringify
#define __stringify_1(x) #x
#define __stringify(x) __stringify_1(x)
#endif

#define HPI_UNUSED(param) (void)param

/** init a buffer that can be sized, from a constant 'C'.
Note that all the sizeof and conditionals are evaluated at
compile time, so this reduces to a memcpy with constant count.
*/
#define HPI_STRCPY_C(dst, src) do { \
	if (sizeof(src) <= sizeof(dst))\
		memcpy(dst, src, sizeof(src));\
	else {\
		memcpy(dst, src, sizeof(dst)-1);\
		dst[sizeof(dst)-1] = 0;\
	} \
} while (0)

/** strncpy with null terminator 'Z' insertion */
#define HPI_STRNCPY_Z(dst, src, dst_len) do { \
	strncpy(dst, src, dst_len);\
	dst[dst_len-1] = 0;\
} while (0)

/* strncpy when dst is char array that gives correct sizeof (not char*) 'SD'
and with null terminator insertion 'Z' */
#define HPI_STRNCPY_SDZ(dst, src) HPI_STRNCPY_Z(dst, src, sizeof(dst))

/* SANITISE DELETE END */

/* If the assert fails, compiler complains
   something like size of array `msg' is negative.
   Unlike linux BUILD_BUG_ON, this works outside function scope.
*/
#define compile_time_assert(cond, msg) \
    typedef char ASSERT_##msg[(cond) ? 1 : -1]

#ifdef __cplusplus
/* *INDENT-OFF* */
extern "C" {
/* *INDENT-ON* */
#endif

#ifndef HPI_OS_LINUX_KERNEL
/*/////////////////////////////////////////////////////////////////////////// */
/* Private HPI Entity related definitions                                     */

#define STR_SIZE_FIELD_MAX 65535U
#define STR_TYPE_FIELD_MAX 255U
#define STR_ROLE_FIELD_MAX 255U

#if defined(_MSC_VER)
#pragma warning( push )
#pragma warning( disable : 4200 )
#endif

struct hpi_entity {
	uint16_t size;
	uint8_t type;
	uint8_t role;
};

#if defined(_MSC_VER)
#pragma warning( pop )
#endif

compile_time_assert((sizeof(struct hpi_entity) == 4), sizeof_hpi_entity);
#endif

/******************************************* bus types */
enum HPI_BUSES {
	HPI_BUS_ISAPNP = 1,
	HPI_BUS_PCI = 2,
	HPI_BUS_USB = 3,
	HPI_BUS_NET = 4
};

enum HPI_SUBSYS_OPTIONS {
	/* 0, 256 are invalid, 1..255 reserved for global options */
	HPI_SUBSYS_OPT_NET_ENABLE = 257,
	HPI_SUBSYS_OPT_NET_BROADCAST = 258,
	HPI_SUBSYS_OPT_NET_UNICAST = 259,
	HPI_SUBSYS_OPT_NET_ADDR = 260,
	HPI_SUBSYS_OPT_NET_MASK = 261,
	HPI_SUBSYS_OPT_NET_ADAPTER_ADDRESS_ADD = 262
};

/** Volume flags
*/
enum HPI_VOLUME_FLAGS {
	/** Set if the volume control is muted */
	HPI_VOLUME_FLAG_MUTED = (1 << 0),
	/** Set if the volume control has a mute function */
	HPI_VOLUME_FLAG_HAS_MUTE = (1 << 1),
	/** Set if volume control can do autofading */
	HPI_VOLUME_FLAG_HAS_AUTOFADE = (1 << 2)
	/* Note Flags >= (1<<8) are for DSP internal use only */
};

/******************************************* CONTROL ATTRIBUTES ****/
/* (in order of control type ID */

/* This allows for 255 control types, 256 unique attributes each */
#define HPI_CTL_ATTR(ctl, ai) ((HPI_CONTROL_##ctl << 8) + ai)

/* Get the sub-index of the attribute for a control type */
#define HPI_CTL_ATTR_INDEX(i) (i & 0xff)

/* Extract the control from the control attribute */
#define HPI_CTL_ATTR_CONTROL(i) (i >> 8)


/** Enable event generation for a control.
0=disable, 1=enable
\note generic to all controls that can generate events
*/

/** Unique identifiers for every control attribute
*/
enum HPI_CONTROL_ATTRIBUTES {
	HPI_GENERIC_ENABLE		= HPI_CTL_ATTR(GENERIC, 1),
	HPI_GENERIC_EVENT_ENABLE	= HPI_CTL_ATTR(GENERIC, 2),

	HPI_VOLUME_GAIN			= HPI_CTL_ATTR(VOLUME, 1),
	HPI_VOLUME_AUTOFADE		= HPI_CTL_ATTR(VOLUME, 2),
	HPI_VOLUME_MUTE			= HPI_CTL_ATTR(VOLUME, 3),
	HPI_VOLUME_GAIN_AND_FLAGS	= HPI_CTL_ATTR(VOLUME, 4),
	HPI_VOLUME_NUM_CHANNELS		= HPI_CTL_ATTR(VOLUME, 6),
	HPI_VOLUME_RANGE		= HPI_CTL_ATTR(VOLUME, 10),

	HPI_METER_RMS			= HPI_CTL_ATTR(METER, 1),
	HPI_METER_PEAK			= HPI_CTL_ATTR(METER, 2),
	HPI_METER_RMS_BALLISTICS	= HPI_CTL_ATTR(METER, 3),
	HPI_METER_PEAK_BALLISTICS	= HPI_CTL_ATTR(METER, 4),
	HPI_METER_NUM_CHANNELS		= HPI_CTL_ATTR(METER, 5),

	HPI_MULTIPLEXER_SOURCE		= HPI_CTL_ATTR(MULTIPLEXER, 1),
	HPI_MULTIPLEXER_QUERYSOURCE	= HPI_CTL_ATTR(MULTIPLEXER, 2),

	HPI_AESEBUTX_FORMAT		= HPI_CTL_ATTR(AESEBUTX, 1),
	HPI_AESEBUTX_SAMPLERATE		= HPI_CTL_ATTR(AESEBUTX, 3),
	HPI_AESEBUTX_CHANNELSTATUS	= HPI_CTL_ATTR(AESEBUTX, 4),
	HPI_AESEBUTX_USERDATA		= HPI_CTL_ATTR(AESEBUTX, 5),

	HPI_AESEBURX_FORMAT		= HPI_CTL_ATTR(AESEBURX, 1),
	HPI_AESEBURX_ERRORSTATUS	= HPI_CTL_ATTR(AESEBURX, 2),
	HPI_AESEBURX_SAMPLERATE		= HPI_CTL_ATTR(AESEBURX, 3),
	HPI_AESEBURX_CHANNELSTATUS	= HPI_CTL_ATTR(AESEBURX, 4),
	HPI_AESEBURX_USERDATA		= HPI_CTL_ATTR(AESEBURX, 5),

	HPI_LEVEL_GAIN			= HPI_CTL_ATTR(LEVEL, 1),
	HPI_LEVEL_RANGE			= HPI_CTL_ATTR(LEVEL, 10),

	HPI_TUNER_BAND			= HPI_CTL_ATTR(TUNER, 1),
	HPI_TUNER_FREQ			= HPI_CTL_ATTR(TUNER, 2),
	HPI_TUNER_LEVEL_AVG		= HPI_CTL_ATTR(TUNER, 3),
	HPI_TUNER_LEVEL_RAW		= HPI_CTL_ATTR(TUNER, 4),
	HPI_TUNER_SNR			= HPI_CTL_ATTR(TUNER, 5),
	HPI_TUNER_GAIN			= HPI_CTL_ATTR(TUNER, 6),
	HPI_TUNER_STATUS		= HPI_CTL_ATTR(TUNER, 7),
	HPI_TUNER_MODE			= HPI_CTL_ATTR(TUNER, 8),
	HPI_TUNER_RDS			= HPI_CTL_ATTR(TUNER, 9),
	HPI_TUNER_DEEMPHASIS		= HPI_CTL_ATTR(TUNER, 10),
	HPI_TUNER_PROGRAM		= HPI_CTL_ATTR(TUNER, 11),
	HPI_TUNER_HDRADIO_SIGNAL_QUALITY = HPI_CTL_ATTR(TUNER, 12),
	HPI_TUNER_HDRADIO_SDK_VERSION	= HPI_CTL_ATTR(TUNER, 13),
	HPI_TUNER_HDRADIO_DSP_VERSION	= HPI_CTL_ATTR(TUNER, 14),
	HPI_TUNER_HDRADIO_BLEND		= HPI_CTL_ATTR(TUNER, 15),

	HPI_VOX_THRESHOLD		= HPI_CTL_ATTR(VOX, 1),

	HPI_CHANNEL_MODE_MODE		= HPI_CTL_ATTR(CHANNEL_MODE, 1),

	HPI_BITSTREAM_DATA_POLARITY	= HPI_CTL_ATTR(BITSTREAM, 1),
	HPI_BITSTREAM_CLOCK_EDGE	= HPI_CTL_ATTR(BITSTREAM, 2),
	HPI_BITSTREAM_CLOCK_SOURCE	= HPI_CTL_ATTR(BITSTREAM, 3),
	HPI_BITSTREAM_ACTIVITY		= HPI_CTL_ATTR(BITSTREAM, 4),

	HPI_SAMPLECLOCK_SOURCE		= HPI_CTL_ATTR(SAMPLECLOCK, 1),
	HPI_SAMPLECLOCK_SAMPLERATE	= HPI_CTL_ATTR(SAMPLECLOCK, 2),
	HPI_SAMPLECLOCK_SOURCE_INDEX	= HPI_CTL_ATTR(SAMPLECLOCK, 3),
	HPI_SAMPLECLOCK_LOCAL_SAMPLERATE = HPI_CTL_ATTR(SAMPLECLOCK, 4),
	HPI_SAMPLECLOCK_AUTO		= HPI_CTL_ATTR(SAMPLECLOCK, 5),
	HPI_SAMPLECLOCK_LOCAL_LOCK	= HPI_CTL_ATTR(SAMPLECLOCK, 6),

	HPI_MICROPHONE_PHANTOM_POWER	= HPI_CTL_ATTR(MICROPHONE, 1),

	HPI_EQUALIZER_NUM_FILTERS	= HPI_CTL_ATTR(EQUALIZER, 1),
	HPI_EQUALIZER_FILTER		= HPI_CTL_ATTR(EQUALIZER, 2),
	HPI_EQUALIZER_COEFFICIENTS	= HPI_CTL_ATTR(EQUALIZER, 3),

	HPI_COMPANDER_PARAMS		= HPI_CTL_ATTR(COMPANDER, 1),
	HPI_COMPANDER_MAKEUPGAIN	= HPI_CTL_ATTR(COMPANDER, 2),
	HPI_COMPANDER_THRESHOLD		= HPI_CTL_ATTR(COMPANDER, 3),
	HPI_COMPANDER_RATIO		= HPI_CTL_ATTR(COMPANDER, 4),
	HPI_COMPANDER_ATTACK		= HPI_CTL_ATTR(COMPANDER, 5),
	HPI_COMPANDER_DECAY		= HPI_CTL_ATTR(COMPANDER, 6),

	HPI_COBRANET_SET		= HPI_CTL_ATTR(COBRANET, 1),
	HPI_COBRANET_GET		= HPI_CTL_ATTR(COBRANET, 2),
	HPI_COBRANET_SET_DATA		= HPI_CTL_ATTR(COBRANET, 3),
	HPI_COBRANET_GET_DATA		= HPI_CTL_ATTR(COBRANET, 4),
	HPI_COBRANET_GET_STATUS		= HPI_CTL_ATTR(COBRANET, 5),
	HPI_COBRANET_SEND_PACKET	= HPI_CTL_ATTR(COBRANET, 6),
	HPI_COBRANET_GET_PACKET		= HPI_CTL_ATTR(COBRANET, 7),

	HPI_TONEDETECTOR_THRESHOLD	= HPI_CTL_ATTR(TONEDETECTOR, 1),
	HPI_TONEDETECTOR_STATE		= HPI_CTL_ATTR(TONEDETECTOR, 2),
	HPI_TONEDETECTOR_FREQUENCY	= HPI_CTL_ATTR(TONEDETECTOR, 3),

	HPI_SILENCEDETECTOR_THRESHOLD	= HPI_CTL_ATTR(SILENCEDETECTOR, 1),
	HPI_SILENCEDETECTOR_STATE 	= HPI_CTL_ATTR(SILENCEDETECTOR, 2),
	HPI_SILENCEDETECTOR_DELAY	= HPI_CTL_ATTR(SILENCEDETECTOR, 3),

	HPI_PAD_CHANNEL_NAME		= HPI_CTL_ATTR(PAD, 1),
	HPI_PAD_ARTIST			= HPI_CTL_ATTR(PAD, 2),
	HPI_PAD_TITLE			= HPI_CTL_ATTR(PAD, 3),
	HPI_PAD_COMMENT			= HPI_CTL_ATTR(PAD, 4),
	HPI_PAD_PROGRAM_TYPE		= HPI_CTL_ATTR(PAD, 5),
	HPI_PAD_PROGRAM_ID		= HPI_CTL_ATTR(PAD, 6),
	HPI_PAD_TA_SUPPORT		= HPI_CTL_ATTR(PAD, 7),
	HPI_PAD_TA_ACTIVE		= HPI_CTL_ATTR(PAD, 8)
};

#define HPI_POLARITY_POSITIVE		0
#define HPI_POLARITY_NEGATIVE		1

/*------------------------------------------------------------
 Cobranet Chip Bridge - copied from HMI.H
------------------------------------------------------------*/
#define  HPI_COBRANET_HMI_cobraBridge		0x20000
#define  HPI_COBRANET_HMI_cobraBridgeTxPktBuf \
	(HPI_COBRANET_HMI_cobraBridge + 0x1000)
#define  HPI_COBRANET_HMI_cobraBridgeRxPktBuf \
	(HPI_COBRANET_HMI_cobraBridge + 0x2000)
#define  HPI_COBRANET_HMI_cobraIfTable1		0x110000
#define  HPI_COBRANET_HMI_cobraIfPhyAddress \
	(HPI_COBRANET_HMI_cobraIfTable1 + 0xd)
#define  HPI_COBRANET_HMI_cobraProtocolIP	0x72000
#define  HPI_COBRANET_HMI_cobraIpMonCurrentIP \
	(HPI_COBRANET_HMI_cobraProtocolIP + 0x0)
#define  HPI_COBRANET_HMI_cobraIpMonStaticIP \
	(HPI_COBRANET_HMI_cobraProtocolIP + 0x2)
#define  HPI_COBRANET_HMI_cobraSys		0x100000
#define  HPI_COBRANET_HMI_cobraSysDesc \
		(HPI_COBRANET_HMI_cobraSys + 0x0)
#define  HPI_COBRANET_HMI_cobraSysObjectID \
	(HPI_COBRANET_HMI_cobraSys + 0x100)
#define  HPI_COBRANET_HMI_cobraSysContact \
	(HPI_COBRANET_HMI_cobraSys + 0x200)
#define  HPI_COBRANET_HMI_cobraSysName \
		(HPI_COBRANET_HMI_cobraSys + 0x300)
#define  HPI_COBRANET_HMI_cobraSysLocation \
	(HPI_COBRANET_HMI_cobraSys + 0x400)

/*------------------------------------------------------------
 Cobranet Chip Status bits
------------------------------------------------------------*/
#define HPI_COBRANET_HMI_STATUS_RXPACKET 2
#define HPI_COBRANET_HMI_STATUS_TXPACKET 3

/*------------------------------------------------------------
 Ethernet header size
------------------------------------------------------------*/
#define HPI_ETHERNET_HEADER_SIZE (16)

/* These defines are used to fill in protocol information for an Ethernet packet
    sent using HMI on CS18102 */
/** ID supplied by Cirrius for ASI packets. */
#define HPI_ETHERNET_PACKET_ID 			0x85
/** Simple packet - no special routing required */
#define HPI_ETHERNET_PACKET_V1 			0x01
/** This packet must make its way to the host across the HPI interface */
#define HPI_ETHERNET_PACKET_HOSTED_VIA_HMI 	0x20
/** This packet must make its way to the host across the HPI interface */
#define HPI_ETHERNET_PACKET_HOSTED_VIA_HMI_V1 	0x21
/** This packet must make its way to the host across the HPI interface */
#define HPI_ETHERNET_PACKET_HOSTED_VIA_HPI 	0x40
/** This packet must make its way to the host across the HPI interface */
#define HPI_ETHERNET_PACKET_HOSTED_VIA_HPI_V1	0x41

#define HPI_ETHERNET_UDP_PORT (44600)	/*!< UDP messaging port */

/** Default network timeout in milli-seconds. */
#define HPI_ETHERNET_TIMEOUT_MS 500

/** Locked memory buffer alloc/free phases */
enum HPI_BUFFER_CMDS {
	/** use one message to allocate or free physical memory */
	HPI_BUFFER_CMD_EXTERNAL = 0,
	/** alloc physical memory */
	HPI_BUFFER_CMD_INTERNAL_ALLOC = 1,
	/** send physical memory address to adapter */
	HPI_BUFFER_CMD_INTERNAL_GRANTADAPTER = 2,
	/** notify adapter to stop using physical buffer */
	HPI_BUFFER_CMD_INTERNAL_REVOKEADAPTER = 3,
	/** free physical buffer */
	HPI_BUFFER_CMD_INTERNAL_FREE = 4
};

/*****************************************************************************/
/*****************************************************************************/
/********		HPI LOW LEVEL MESSAGES			*******/
/*****************************************************************************/
/*****************************************************************************/
/** Pnp ids */
/** "ASI"  - actual is "ASX" - need to change */
#define HPI_ID_ISAPNP_AUDIOSCIENCE	0x0669
/** PCI vendor ID that AudioScience uses */
#define HPI_PCI_VENDOR_ID_AUDIOSCIENCE	0x175C
/** PCI vendor ID that the DSP56301 has */
#define HPI_PCI_VENDOR_ID_MOTOROLA	0x1057
/** PCI vendor ID that TI uses */
#define HPI_PCI_VENDOR_ID_TI		0x104C

#define HPI_PCI_DEV_ID_PCI2040		0xAC60
/** TI's C6205 PCI interface has this ID */
#define HPI_PCI_DEV_ID_DSP6205		0xA106

#define HPI_USB_VENDOR_ID_AUDIOSCIENCE	0x1257
#define HPI_USB_W2K_TAG			0x57495341	/* "ASIW"       */
#define HPI_USB_LINUX_TAG		0x4C495341	/* "ASIL"       */

/** Invalid Adapter index
Used in HPI messages that are not addressed to a specific adapter
Used in DLL to indicate device not present
*/
#define HPI_ADAPTER_INDEX_INVALID 0xFFFF

/** First 2 hex digits define the adapter family */
#define HPI_ADAPTER_FAMILY_MASK		0xff00
#define HPI_MODULE_FAMILY_MASK		0xfff0

#define HPI_ADAPTER_FAMILY_ASI(f)   (f & HPI_ADAPTER_FAMILY_MASK)
#define HPI_MODULE_FAMILY_ASI(f)   (f & HPI_MODULE_FAMILY_MASK)
#define HPI_ADAPTER_ASI(f)   (f)

enum HPI_MESSAGE_TYPES {
	HPI_TYPE_MESSAGE		= 1,
	HPI_TYPE_RESPONSE		= 2,
	HPI_TYPE_DATA			= 3,
	HPI_TYPE_SSX2BYPASS_MESSAGE	= 4
};

enum HPI_OBJECT_TYPES {
	HPI_OBJ_SUBSYSTEM	= 1,
	HPI_OBJ_ADAPTER		= 2,
	HPI_OBJ_OSTREAM		= 3,
	HPI_OBJ_ISTREAM		= 4,
	HPI_OBJ_MIXER		= 5,
	HPI_OBJ_NODE		= 6,
	HPI_OBJ_CONTROL		= 7,
	HPI_OBJ_NVMEMORY	= 8,
	HPI_OBJ_GPIO		= 9,
	HPI_OBJ_WATCHDOG	= 10,
	HPI_OBJ_CLOCK		= 11,
	HPI_OBJ_PROFILE		= 12,
	HPI_OBJ_CONTROLEX	= 13,
	HPI_OBJ_ASYNCEVENT	= 14

	#define HPI_OBJ_MAXINDEX 14
};

#define HPI_OBJ_FUNCTION_SPACING 0x100
#define HPI_FUNC_ID(obj, i) (HPI_OBJ_##obj * HPI_OBJ_FUNCTION_SPACING + i)

#define HPI_EXTRACT_INDEX(fn) (fn & 0xff)

enum HPI_FUNCTION_IDS {
	HPI_SUBSYS_OPEN			= HPI_FUNC_ID(SUBSYSTEM, 1),
	HPI_SUBSYS_GET_VERSION		= HPI_FUNC_ID(SUBSYSTEM, 2),
	HPI_SUBSYS_GET_INFO		= HPI_FUNC_ID(SUBSYSTEM, 3),
	HPI_SUBSYS_FIND_ADAPTERS	= HPI_FUNC_ID(SUBSYSTEM, 4),
	HPI_SUBSYS_CREATE_ADAPTER	= HPI_FUNC_ID(SUBSYSTEM, 5),
	HPI_SUBSYS_CLOSE		= HPI_FUNC_ID(SUBSYSTEM, 6),
	HPI_SUBSYS_DELETE_ADAPTER	= HPI_FUNC_ID(SUBSYSTEM, 7),
	HPI_SUBSYS_DRIVER_LOAD		= HPI_FUNC_ID(SUBSYSTEM, 8),
	HPI_SUBSYS_DRIVER_UNLOAD	= HPI_FUNC_ID(SUBSYSTEM, 9),
	HPI_SUBSYS_READ_PORT_8		= HPI_FUNC_ID(SUBSYSTEM, 10),
	HPI_SUBSYS_WRITE_PORT_8		= HPI_FUNC_ID(SUBSYSTEM, 11),
	HPI_SUBSYS_GET_NUM_ADAPTERS	= HPI_FUNC_ID(SUBSYSTEM, 12),
	HPI_SUBSYS_GET_ADAPTER		= HPI_FUNC_ID(SUBSYSTEM, 13),
	HPI_SUBSYS_SET_NETWORK_INTERFACE = HPI_FUNC_ID(SUBSYSTEM, 14),
	HPI_SUBSYS_OPTION_INFO = HPI_FUNC_ID(SUBSYSTEM, 15),
	HPI_SUBSYS_OPTION_GET = HPI_FUNC_ID(SUBSYSTEM, 16),
	HPI_SUBSYS_OPTION_SET = HPI_FUNC_ID(SUBSYSTEM, 17),
	#define HPI_SUBSYS_FUNCTION_COUNT 17

	HPI_ADAPTER_OPEN		= HPI_FUNC_ID(ADAPTER, 1),
	HPI_ADAPTER_CLOSE		= HPI_FUNC_ID(ADAPTER, 2),
	HPI_ADAPTER_GET_INFO		= HPI_FUNC_ID(ADAPTER, 3),
	HPI_ADAPTER_GET_ASSERT		= HPI_FUNC_ID(ADAPTER, 4),
	HPI_ADAPTER_TEST_ASSERT		= HPI_FUNC_ID(ADAPTER, 5),
	HPI_ADAPTER_SET_MODE		= HPI_FUNC_ID(ADAPTER, 6),
	HPI_ADAPTER_GET_MODE		= HPI_FUNC_ID(ADAPTER, 7),
	HPI_ADAPTER_ENABLE_CAPABILITY	= HPI_FUNC_ID(ADAPTER, 8),
	HPI_ADAPTER_SELFTEST		= HPI_FUNC_ID(ADAPTER, 9),
	HPI_ADAPTER_FIND_OBJECT		= HPI_FUNC_ID(ADAPTER, 10),
	HPI_ADAPTER_QUERY_FLASH		= HPI_FUNC_ID(ADAPTER, 11),
	HPI_ADAPTER_START_FLASH		= HPI_FUNC_ID(ADAPTER, 12),
	HPI_ADAPTER_PROGRAM_FLASH	= HPI_FUNC_ID(ADAPTER, 13),
	HPI_ADAPTER_SET_PROPERTY	= HPI_FUNC_ID(ADAPTER, 14),
	HPI_ADAPTER_GET_PROPERTY	= HPI_FUNC_ID(ADAPTER, 15),
	HPI_ADAPTER_ENUM_PROPERTY	= HPI_FUNC_ID(ADAPTER, 16),
	HPI_ADAPTER_MODULE_INFO		= HPI_FUNC_ID(ADAPTER, 17),
	HPI_ADAPTER_DEBUG_READ		= HPI_FUNC_ID(ADAPTER, 18),
	HPI_ADAPTER_IRQ_QUERY_AND_CLEAR = HPI_FUNC_ID(ADAPTER, 19),
	HPI_ADAPTER_IRQ_CALLBACK	= HPI_FUNC_ID(ADAPTER, 20),
	#define HPI_ADAPTER_FUNCTION_COUNT 20

	HPI_OSTREAM_OPEN		= HPI_FUNC_ID(OSTREAM, 1),
	HPI_OSTREAM_CLOSE		= HPI_FUNC_ID(OSTREAM, 2),
	HPI_OSTREAM_WRITE		= HPI_FUNC_ID(OSTREAM, 3),
	HPI_OSTREAM_START		= HPI_FUNC_ID(OSTREAM, 4),
	HPI_OSTREAM_STOP		= HPI_FUNC_ID(OSTREAM, 5),
	HPI_OSTREAM_RESET		= HPI_FUNC_ID(OSTREAM, 6),
	HPI_OSTREAM_GET_INFO		= HPI_FUNC_ID(OSTREAM, 7),
	HPI_OSTREAM_QUERY_FORMAT	= HPI_FUNC_ID(OSTREAM, 8),
	HPI_OSTREAM_DATA		= HPI_FUNC_ID(OSTREAM, 9),
	HPI_OSTREAM_SET_VELOCITY	= HPI_FUNC_ID(OSTREAM, 10),
	HPI_OSTREAM_SET_PUNCHINOUT	= HPI_FUNC_ID(OSTREAM, 11),
	HPI_OSTREAM_SINEGEN		= HPI_FUNC_ID(OSTREAM, 12),
	HPI_OSTREAM_ANC_RESET		= HPI_FUNC_ID(OSTREAM, 13),
	HPI_OSTREAM_ANC_GET_INFO	= HPI_FUNC_ID(OSTREAM, 14),
	HPI_OSTREAM_ANC_READ		= HPI_FUNC_ID(OSTREAM, 15),
	HPI_OSTREAM_SET_TIMESCALE	= HPI_FUNC_ID(OSTREAM, 16),
	HPI_OSTREAM_SET_FORMAT		= HPI_FUNC_ID(OSTREAM, 17),
	HPI_OSTREAM_HOSTBUFFER_ALLOC	= HPI_FUNC_ID(OSTREAM, 18),
	HPI_OSTREAM_HOSTBUFFER_FREE	= HPI_FUNC_ID(OSTREAM, 19),
	HPI_OSTREAM_GROUP_ADD		= HPI_FUNC_ID(OSTREAM, 20),
	HPI_OSTREAM_GROUP_GETMAP	= HPI_FUNC_ID(OSTREAM, 21),
	HPI_OSTREAM_GROUP_RESET		= HPI_FUNC_ID(OSTREAM, 22),
	HPI_OSTREAM_HOSTBUFFER_GET_INFO	= HPI_FUNC_ID(OSTREAM, 23),
	HPI_OSTREAM_WAIT_START		= HPI_FUNC_ID(OSTREAM, 24),
	HPI_OSTREAM_WAIT		= HPI_FUNC_ID(OSTREAM, 25),
	#define HPI_OSTREAM_FUNCTION_COUNT 25

	HPI_ISTREAM_OPEN		= HPI_FUNC_ID(ISTREAM, 1),
	HPI_ISTREAM_CLOSE		= HPI_FUNC_ID(ISTREAM, 2),
	HPI_ISTREAM_SET_FORMAT		= HPI_FUNC_ID(ISTREAM, 3),
	HPI_ISTREAM_READ		= HPI_FUNC_ID(ISTREAM, 4),
	HPI_ISTREAM_START		= HPI_FUNC_ID(ISTREAM, 5),
	HPI_ISTREAM_STOP		= HPI_FUNC_ID(ISTREAM, 6),
	HPI_ISTREAM_RESET		= HPI_FUNC_ID(ISTREAM, 7),
	HPI_ISTREAM_GET_INFO		= HPI_FUNC_ID(ISTREAM, 8),
	HPI_ISTREAM_QUERY_FORMAT	= HPI_FUNC_ID(ISTREAM, 9),
	HPI_ISTREAM_ANC_RESET		= HPI_FUNC_ID(ISTREAM, 10),
	HPI_ISTREAM_ANC_GET_INFO	= HPI_FUNC_ID(ISTREAM, 11),
	HPI_ISTREAM_ANC_WRITE		= HPI_FUNC_ID(ISTREAM, 12),
	HPI_ISTREAM_HOSTBUFFER_ALLOC	= HPI_FUNC_ID(ISTREAM, 13),
	HPI_ISTREAM_HOSTBUFFER_FREE	= HPI_FUNC_ID(ISTREAM, 14),
	HPI_ISTREAM_GROUP_ADD		= HPI_FUNC_ID(ISTREAM, 15),
	HPI_ISTREAM_GROUP_GETMAP	= HPI_FUNC_ID(ISTREAM, 16),
	HPI_ISTREAM_GROUP_RESET		= HPI_FUNC_ID(ISTREAM, 17),
	HPI_ISTREAM_HOSTBUFFER_GET_INFO	= HPI_FUNC_ID(ISTREAM, 18),
	HPI_ISTREAM_WAIT_START		= HPI_FUNC_ID(ISTREAM, 19),
	HPI_ISTREAM_WAIT		= HPI_FUNC_ID(ISTREAM, 20),
	#define HPI_ISTREAM_FUNCTION_COUNT 20

/* NOTE:
   GET_NODE_INFO, SET_CONNECTION, GET_CONNECTIONS are not currently used */
	HPI_MIXER_OPEN			= HPI_FUNC_ID(MIXER, 1),
	HPI_MIXER_CLOSE			= HPI_FUNC_ID(MIXER, 2),
	HPI_MIXER_GET_INFO		= HPI_FUNC_ID(MIXER, 3),
	HPI_MIXER_GET_NODE_INFO		= HPI_FUNC_ID(MIXER, 4),
	HPI_MIXER_GET_CONTROL		= HPI_FUNC_ID(MIXER, 5),
	HPI_MIXER_SET_CONNECTION	= HPI_FUNC_ID(MIXER, 6),
	HPI_MIXER_GET_CONNECTIONS	= HPI_FUNC_ID(MIXER, 7),
	HPI_MIXER_GET_CONTROL_BY_INDEX	= HPI_FUNC_ID(MIXER, 8),
	HPI_MIXER_GET_CONTROL_ARRAY_BY_INDEX = HPI_FUNC_ID(MIXER, 9),
	HPI_MIXER_GET_CONTROL_MULTIPLE_VALUES = HPI_FUNC_ID(MIXER, 10),
	HPI_MIXER_STORE			= HPI_FUNC_ID(MIXER, 11),
	HPI_MIXER_GET_CACHE_INFO	= HPI_FUNC_ID(MIXER, 12),
	#define HPI_MIXER_FUNCTION_COUNT 12

	HPI_CONTROL_GET_INFO		= HPI_FUNC_ID(CONTROL, 1),
	HPI_CONTROL_GET_STATE		= HPI_FUNC_ID(CONTROL, 2),
	HPI_CONTROL_SET_STATE		= HPI_FUNC_ID(CONTROL, 3),
	#define HPI_CONTROL_FUNCTION_COUNT 3

	HPI_NVMEMORY_OPEN		= HPI_FUNC_ID(NVMEMORY, 1),
	HPI_NVMEMORY_READ_BYTE		= HPI_FUNC_ID(NVMEMORY, 2),
	HPI_NVMEMORY_WRITE_BYTE		= HPI_FUNC_ID(NVMEMORY, 3),
	#define HPI_NVMEMORY_FUNCTION_COUNT 3

	HPI_GPIO_OPEN			= HPI_FUNC_ID(GPIO, 1),
	HPI_GPIO_READ_BIT		= HPI_FUNC_ID(GPIO, 2),
	HPI_GPIO_WRITE_BIT		= HPI_FUNC_ID(GPIO, 3),
	HPI_GPIO_READ_ALL		= HPI_FUNC_ID(GPIO, 4),
	HPI_GPIO_WRITE_STATUS		= HPI_FUNC_ID(GPIO, 5),
	#define HPI_GPIO_FUNCTION_COUNT 5

	HPI_ASYNCEVENT_OPEN		= HPI_FUNC_ID(ASYNCEVENT, 1),
	HPI_ASYNCEVENT_CLOSE		= HPI_FUNC_ID(ASYNCEVENT, 2),
	HPI_ASYNCEVENT_WAIT		= HPI_FUNC_ID(ASYNCEVENT, 3),
	HPI_ASYNCEVENT_GETCOUNT		= HPI_FUNC_ID(ASYNCEVENT, 4),
	HPI_ASYNCEVENT_GET		= HPI_FUNC_ID(ASYNCEVENT, 5),
	HPI_ASYNCEVENT_SENDEVENTS	= HPI_FUNC_ID(ASYNCEVENT, 6),
	#define HPI_ASYNCEVENT_FUNCTION_COUNT 6

	HPI_WATCHDOG_OPEN		= HPI_FUNC_ID(WATCHDOG, 1),
	HPI_WATCHDOG_SET_TIME		= HPI_FUNC_ID(WATCHDOG, 2),
	HPI_WATCHDOG_PING		= HPI_FUNC_ID(WATCHDOG, 3),

	HPI_CLOCK_OPEN			= HPI_FUNC_ID(CLOCK, 1),
	HPI_CLOCK_SET_TIME		= HPI_FUNC_ID(CLOCK, 2),
	HPI_CLOCK_GET_TIME		= HPI_FUNC_ID(CLOCK, 3),

	HPI_PROFILE_OPEN_ALL		= HPI_FUNC_ID(PROFILE, 1),
	HPI_PROFILE_START_ALL		= HPI_FUNC_ID(PROFILE, 2),
	HPI_PROFILE_STOP_ALL		= HPI_FUNC_ID(PROFILE, 3),
	HPI_PROFILE_GET			= HPI_FUNC_ID(PROFILE, 4),
	HPI_PROFILE_GET_IDLECOUNT	= HPI_FUNC_ID(PROFILE, 5),
	HPI_PROFILE_GET_NAME		= HPI_FUNC_ID(PROFILE, 6),
	HPI_PROFILE_GET_UTILIZATION	= HPI_FUNC_ID(PROFILE, 7)
	#define HPI_PROFILE_FUNCTION_COUNT 7
};

/* ////////////////////////////////////////////////////////////////////// */
/* STRUCTURES */
#ifndef DISABLE_PRAGMA_PACK1
#pragma pack(push, 1)
#endif

/** PCI bus resource */
struct hpi_pci {
	uint32_t __iomem *apMemBase[HPI_MAX_ADAPTER_MEM_SPACES];
	struct pci_dev *pOsData;
};

struct hpi_resource {
	union {
		const struct hpi_pci *pci;
		const char *net_if;
	} r;
#ifndef HPI_BUILD_64BIT	/* keep structure size constant */
	uint32_t dwPadTo64;
#endif
	uint16_t wBusType;	/* HPI_BUS_PNPISA, _PCI, _USB etc */
	uint16_t wPadding;

};

/** Format info used inside struct hpi_message
    Not the same as public API struct hpi_format */
struct hpi_msg_format {
	uint32_t dwSampleRate; /**< 11025, 32000, 44100 etc. */
	uint32_t dwBitRate; /**< for MPEG */
	uint32_t dwAttributes; /**< Stereo/JointStereo/Mono */
	uint16_t wChannels; /**< 1,2..., (or ancillary mode or idle bit */
	uint16_t wFormat; /**< HPI_FORMAT_PCM16, _MPEG etc. see \ref HPI_FORMATS. */
};

/**  Buffer+format structure.
	 Must be kept 7 * 32 bits to match public HPI_DATA struct */
struct hpi_msg_data {
	struct hpi_msg_format format;
	uint8_t *pbData;
#ifndef HPI_BUILD_64BIT
	uint32_t dwPadding;
#endif
	uint32_t dwDataSize;
};

#ifndef HPI_ON_DSP
/** HPI_DATA structure used up to 3.04 driver */
struct hpi_data_legacy32 {
	struct hpi_format format;
	uint32_t pbData;
	uint32_t dwDataSize;
};
#endif

#ifdef HPI_BUILD_64BIT
/* Compatibility version of HPI_DATA */
struct hpi_data_compat32 {
	struct hpi_msg_format format;
	uint32_t pbData;
	uint32_t dwPadding;
	uint32_t dwDataSize;
};
#endif

struct hpi_buffer {
  /** placehoder for backward compatability (see dwBufferSize) */
	struct hpi_msg_format reserved;
	uint32_t dwCommand; /**< HPI_BUFFER_CMD_xxx*/
	uint32_t dwPciAddress; /**< PCI physical address of buffer for DSP DMA */
	uint32_t dwBufferSize; /**< must line up with dwDataSize of HPI_DATA*/
};

/*/////////////////////////////////////////////////////////////////////////// */
/* This is used for background buffer bus mastering stream buffers.           */
struct hpi_hostbuffer_status {
	uint32_t dwSamplesProcessed;
	uint32_t dwAuxiliaryDataAvailable;
	uint32_t dwStreamState;
	// DSP index in to the host bus master buffer.
	uint32_t dwDSPIndex;
	// Host index in to the host bus master buffer.
	uint32_t dwHostIndex;
	uint32_t dwSizeInBytes;
};

struct hpi_streamid {
	uint16_t wObjectType;
		    /**< Type of object, HPI_OBJ_OSTREAM or HPI_OBJ_ISTREAM. */
	uint16_t wStreamIndex; /**< OStream or IStream index. */
};

struct hpi_punchinout {
	uint32_t dwPunchInSample;
	uint32_t dwPunchOutSample;
};

struct hpi_subsys_msg {
	struct hpi_resource resource;
};

#ifdef HPI_SUBSYS_RES_NO_LIST
struct hpi_subsys_res {
	uint32_t dwVersion;
	uint32_t dwData; /* extended version */
	uint16_t wNumAdapters;
	uint16_t wAdapterIndex;
	uint16_t wAdapterType;
	uint16_t pad16;
};
#else
struct hpi_subsys_res {
	uint32_t dwVersion;
	uint32_t dwData;		/* used to return extended version */
	uint16_t wNumAdapters;	/* number of adapters */
	uint16_t wAdapterIndex;
	uint16_t awAdapterList[HPI_MAX_ADAPTERS];
};
#endif

union hpi_adapterx_msg {
	struct {
		uint32_t dwDspAddress;
		uint32_t dwCountBytes;
	} debug_read;
	struct {
		uint32_t adapter_mode;
		uint16_t query_or_set;
	} mode;
	struct {
		uint16_t index;
	} module_info;
	struct {
		uint32_t dwChecksum;
		uint16_t wSequence;
		uint16_t wLength;
		uint16_t wOffset; /**< offset from start of msg to data */
		uint16_t wUnused;
	} program_flash;
	struct {
		uint16_t wIndex;
		uint16_t wWhat;
		uint16_t wPropertyIndex;
	} property_enum;
	struct {
		uint16_t wProperty;
		uint16_t wParameter1;
		uint16_t wParameter2;
	} property_set;
	struct {
		uint32_t dwOffset;
	} query_flash;
	struct {
		uint32_t pad32;
		uint16_t key1;
		uint16_t key2;
	} restart;
	struct {
		uint32_t dwOffset;
		uint32_t dwLength;
		uint32_t dwKey;
	} start_flash;
	struct {
		uint32_t pad32;
		uint16_t value;
	} test_assert;
	struct {
		uint32_t yes ;
	} irq_query;
};

struct hpi_adapter_res {
	uint32_t dwSerialNumber;
	uint16_t wAdapterType;
	uint16_t wAdapterIndex;
	uint16_t wNumIStreams;
	uint16_t wNumOStreams;
	uint16_t wNumMixers;
	uint16_t wVersion;
	uint8_t szAdapterAssert[HPI_STRING_LEN];
};

union hpi_adapterx_res {
	struct  hpi_adapter_res info;
	struct {
		uint32_t p1;
		uint16_t count;
		uint16_t dsp_index;
		uint32_t p2;
		uint32_t dsp_msg_addr;
		char szMessage[HPI_STRING_LEN];
	} assert;
	struct {
		uint32_t adapter_mode;
	} mode;
	struct {
		uint16_t wSequence;
	} program_flash;
	struct {
		uint16_t wParameter1;
		uint16_t wParameter2;
	} property_get;
	struct {
		uint32_t dwChecksum;
		uint32_t dwLength;
		uint32_t dwVersion;
	} query_flash;
	struct {
		uint32_t yes;
	} irq_query;
};

struct hpi_stream_msg {
	union {
		struct hpi_msg_data data;
#ifndef HPI_ON_DSP
		struct hpi_data_legacy32 data32;
#endif
		uint16_t wVelocity;
		struct hpi_punchinout pio;
		uint32_t dwTimeScale;
		struct hpi_buffer buffer;
		struct hpi_streamid stream;
		uint32_t dwThresholdBytes;
	} u;
};

struct hpi_stream_res {
	union {
		struct {
			/* size of hardware buffer */
			uint32_t dwBufferSize;
			/* OutStream - data to play,
			   InStream - data recorded */
			uint32_t dwDataAvailable;
			/* OutStream - samples played,
			   InStream - samples recorded */
			uint32_t dwSamplesTransferred;
			/* Adapter - OutStream - data to play,
			   InStream - data recorded */
			uint32_t dwAuxiliaryDataAvailable;
			uint16_t wState;	/* HPI_STATE_PLAYING, _STATE_STOPPED */
			uint16_t wPadding;
		} stream_info;
		struct {
			uint32_t dwBufferSize;
			uint32_t dwDataAvailable;
			uint32_t dwSamplesTransfered;
			uint16_t wState;
			uint16_t wOStreamIndex;
			uint16_t wIStreamIndex;
			uint16_t wPadding;
			uint32_t dwAuxiliaryDataAvailable;
		} legacy_stream_info;
		struct {
			/* bitmap of grouped OutStreams */
			uint32_t dwOutStreamGroupMap;
			/* bitmap of grouped InStreams */
			uint32_t dwInStreamGroupMap;
		} group_info;
		struct {
			/* pointer to the buffer */
			uint8_t *pBuffer;
			/* pointer to the hostbuffer status */
			struct hpi_hostbuffer_status *pStatus;
		} hostbuffer_info;
	} u;
};

struct hpi_mixer_msg {
	uint16_t wControlIndex;
	uint16_t wControlType;	/* = HPI_CONTROL_METER _VOLUME etc */
	uint16_t wPadding1;		/* Maintain alignment of subsequent fields */
	uint16_t wNodeType1;	/* = HPI_SOURCENODE_LINEIN etc */
	uint16_t wNodeIndex1;	/* = 0..N */
	uint16_t wNodeType2;
	uint16_t wNodeIndex2;
	uint16_t wPadding2;		/* round to 4 bytes */
};

struct hpi_mixer_res {
	uint16_t wSrcNodeType;	/* = HPI_SOURCENODE_LINEIN etc */
	uint16_t wSrcNodeIndex;	/* = 0..N */
	uint16_t wDstNodeType;
	uint16_t wDstNodeIndex;
	/* Also controlType for MixerGetControlByIndex */
	uint16_t wControlIndex;
	/* may indicate which DSP the control is located on */
	uint16_t wDspIndex;
};

union hpi_mixerx_msg {
	struct {
		uint16_t wStartingIndex;
		uint16_t wFlags;
		uint32_t dwLengthInBytes;	/* length in bytes of pData */
		uint32_t pData;	/* pointer to a data array */
	} gcabi;
	struct {
		uint16_t wCommand;
		uint16_t wIndex;
	} store;		/* for HPI_MIXER_STORE message */
};

union hpi_mixerx_res {
	struct {
		uint32_t dwBytesReturned;	/* size of items returned */
		uint32_t pData;	/* pointer to data array */
		uint16_t wMoreToDo;	/* indicates if there is more to do */
	} gcabi;
	struct {
		uint32_t total_controls; /* count of controls in the mixer */
		uint32_t cache_controls; /* count of controls in the cac */
		uint32_t cache_bytes;	/* size of cache */
	} cache_info;
};

struct hpi_control_msg {
	uint16_t wAttribute;	/* control attribute or property */
	uint16_t wSavedIndex;
	uint32_t dwParam1;		/* generic parameter 1 */
	uint32_t dwParam2;		/* generic parameter 2 */
	short anLogValue[HPI_MAX_CHANNELS];
};

struct hpi_control_union_msg {
	uint16_t wAttribute;	/* control attribute or property */
	uint16_t wSavedIndex;	/* Only used in ctrl save/restore */
	union {
		struct {
			uint32_t dwParam1;	/* generic parameter 1 */
			uint32_t dwParam2;	/* generic parameter 2 */
			short anLogValue[HPI_MAX_CHANNELS];
		} old;
		union {
			uint32_t dwFrequency;
			uint32_t dwGain;
			uint32_t dwBand;
			uint32_t dwDeemphasis;
			uint32_t dwProgram;
			struct {
				uint32_t dwMode;
				uint32_t dwValue;
			} mode;
			uint32_t dwBlend;
		} tuner;
	} u;
};

struct hpi_control_res {
	/* Could make union. dwParam, anLogValue never used in same response */
	uint32_t dwParam1;
	uint32_t dwParam2;
	short anLogValue[HPI_MAX_CHANNELS];
};

union hpi_control_union_res {
	struct {
		uint32_t dwParam1;
		uint32_t dwParam2;
		short anLogValue[HPI_MAX_CHANNELS];
	} old;
	union {
		uint32_t dwBand;
		uint32_t dwFrequency;
		uint32_t dwGain;
		uint32_t dwDeemphasis;
		struct {
			uint32_t dwData[2];
			uint32_t dwBLER;
		} rds;
		short sLevel;
		struct {
			uint16_t value;
			uint16_t mask;
		} status;
	} tuner;
	struct {
		char szData[8];
		uint32_t dwRemainingChars;
	} chars8;
	char cData12[12];
};

/* HPI_CONTROLX_STRUCTURES */

/* Message */

/** Used for all HMI variables where max length <= 8 bytes
*/
struct hpi_controlx_msg_cobranet_data {
	uint32_t dwHmiAddress;
	uint32_t dwByteCount;
	uint32_t dwData[2];
};

/** Used for string data, and for packet bridge
*/
struct hpi_controlx_msg_cobranet_bigdata {
	uint32_t dwHmiAddress;
	uint32_t dwByteCount;
	uint8_t *pbData;
#ifndef HPI_BUILD_64BIT
	uint32_t dwPadding;
#endif
};

/** Used for PADS control reading of string fields.
*/
struct hpi_controlx_msg_pad_data {
	uint32_t dwField;
	uint32_t dwByteCount;
	uint8_t *pbData;
#ifndef HPI_BUILD_64BIT
	uint32_t dwPadding;
#endif
};

/** Used for generic data
*/

struct hpi_controlx_msg_generic {
	uint32_t dwParam1;
	uint32_t dwParam2;
};

struct hpi_controlx_msg {
	uint16_t wAttribute;	/* control attribute or property */
	uint16_t wSavedIndex;
	union {
		struct hpi_controlx_msg_cobranet_data cobranet_data;
		struct hpi_controlx_msg_cobranet_bigdata cobranet_bigdata;
		struct hpi_controlx_msg_generic generic;
		struct hpi_controlx_msg_pad_data pad_data;
		//struct param_value universal_value;
		/* nothing extra to send for status read */
	} u;
};

/* Response */
/**
*/
struct hpi_controlx_res_cobranet_data {
	uint32_t dwByteCount;
	uint32_t dwData[2];
};

struct hpi_controlx_res_cobranet_bigdata {
	uint32_t dwByteCount;
};

struct hpi_controlx_res_cobranet_status {
	uint32_t dwStatus;
	uint32_t dwReadableSize;
	uint32_t dwWriteableSize;
};

struct hpi_controlx_res_generic {
	uint32_t dwParam1;
	uint32_t dwParam2;
};

struct hpi_controlx_res {
	union {
		struct hpi_controlx_res_cobranet_bigdata cobranet_bigdata;
		struct hpi_controlx_res_cobranet_data cobranet_data;
		struct hpi_controlx_res_cobranet_status cobranet_status;
		struct hpi_controlx_res_generic generic;
		//struct param_info universal_info;
		//struct param_value universal_value;
	} u;
};

struct hpi_nvmemory_msg {
	uint16_t wAddress;
	uint16_t wData;
};

struct hpi_nvmemory_res {
	uint16_t wSizeInBytes;
	uint16_t wData;
};

struct hpi_gpio_msg {
	uint16_t wBitIndex;
	uint16_t wBitData;
};

struct hpi_gpio_res {
	uint16_t wNumberInputBits;
	uint16_t wNumberOutputBits;
	uint16_t wBitData[4];
};

struct hpi_async_msg {
	uint32_t dwEvents;
	uint16_t wMaximumEvents;
	uint16_t wPadding;
};

struct hpi_async_res {
	union {
		struct {
			uint16_t wCount;
		} count;
		struct {
			uint32_t dwEvents;
			uint16_t wNumberReturned;
			uint16_t wPadding;
		} get;
		struct hpi_async_event event;
	} u;
};

struct hpi_watchdog_msg {
	uint32_t dwTimeMs;
};

struct hpi_watchdog_res {
	uint32_t dwTimeMs;
};

struct hpi_clock_msg {
	uint16_t wHours;
	uint16_t wMinutes;
	uint16_t wSeconds;
	uint16_t wMilliSeconds;
};

struct hpi_clock_res {
	uint16_t wSizeInBytes;
	uint16_t wHours;
	uint16_t wMinutes;
	uint16_t wSeconds;
	uint16_t wMilliSeconds;
	uint16_t wPadding;
};

struct hpi_profile_msg {
	uint16_t wBinIndex;
	uint16_t wPadding;
};

struct hpi_profile_res_open {
	uint16_t wMaxProfiles;
};

struct hpi_profile_res_time {
	uint32_t dwTotalTickCount;
	uint32_t dwCallCount;
	uint32_t dwMaxTickCount;
	uint32_t dwTicksPerMillisecond;
	uint16_t wProfileInterval;
};

struct hpi_profile_res_name {
	uint8_t szName[32];
};

struct hpi_profile_res {
	union {
		struct hpi_profile_res_open o;
		struct hpi_profile_res_time t;
		struct hpi_profile_res_name n;
	} u;
};

struct hpi_message_header {
	uint16_t wSize;		/* total size in bytes */
	uint8_t wType;		/* HPI_TYPE_MESSAGE  */
	uint8_t version;		/* message version */
	uint16_t wObject;		/* HPI_OBJ_* */
	uint16_t wFunction;		/* HPI_SUBSYS_xxx, HPI_ADAPTER_xxx */
	uint16_t wAdapterIndex;	/* the adapter index */
	uint16_t wObjIndex;		/* */
};

struct hpi_message {
#ifdef HPI_HEADER_AS_STRUCT
	struct hpi_message_header h;
#else
	/* following fields must match HPI_MESSAGE_HEADER */
	uint16_t wSize;		/* total size in bytes */
	uint8_t wType;		/* HPI_TYPE_MESSAGE  */
	uint8_t version;		/* message version */
	uint16_t wObject;		/* HPI_OBJ_* */
	uint16_t wFunction;		/* HPI_SUBSYS_xxx, HPI_ADAPTER_xxx */
	uint16_t wAdapterIndex;	/* the adapter index */
	uint16_t wObjIndex;		/*  */
#endif
	union {
		struct hpi_subsys_msg s;
		union hpi_adapterx_msg ax;
		struct hpi_stream_msg d;
		struct hpi_mixer_msg m;
		union hpi_mixerx_msg mx;	/* extended mixer; */
		struct hpi_control_msg c;	/* mixer control; */
		/* identical to struct hpi_control_msg,
		   but field naming is improved */
		struct hpi_control_union_msg cu;
		struct hpi_controlx_msg cx;	/* extended mixer control; */
		struct hpi_nvmemory_msg n;
		struct hpi_gpio_msg l;	/* digital i/o */
		struct hpi_watchdog_msg w;
		struct hpi_clock_msg t;	/* dsp time */
		struct hpi_profile_msg p;
		struct hpi_async_msg as;
		char   fixed_size[32];
	} u;
};

#define HPI_MESSAGE_SIZE_BY_OBJECT { \
	sizeof(struct hpi_message_header) ,   /* Default, no object type 0 */ \
	sizeof(struct hpi_message_header) + sizeof(struct hpi_subsys_msg),\
	sizeof(struct hpi_message_header) + sizeof(union hpi_adapterx_msg),\
	sizeof(struct hpi_message_header) + sizeof(struct hpi_stream_msg),\
	sizeof(struct hpi_message_header) + sizeof(struct hpi_stream_msg),\
	sizeof(struct hpi_message_header) + sizeof(struct hpi_mixer_msg),\
	sizeof(struct hpi_message_header) ,   /* no node message */ \
	sizeof(struct hpi_message_header) + sizeof(struct hpi_control_msg),\
	sizeof(struct hpi_message_header) + sizeof(struct hpi_nvmemory_msg),\
	sizeof(struct hpi_message_header) + sizeof(struct hpi_gpio_msg),\
	sizeof(struct hpi_message_header) + sizeof(struct hpi_watchdog_msg),\
	sizeof(struct hpi_message_header) + sizeof(struct hpi_clock_msg),\
	sizeof(struct hpi_message_header) + sizeof(struct hpi_profile_msg),\
	sizeof(struct hpi_message_header) + sizeof(struct hpi_controlx_msg),\
	sizeof(struct hpi_message_header) + sizeof(struct hpi_async_msg) \
}

/*
Note that the wSpecificError error field should be inspected and potentially
reported whenever HPI_ERROR_DSP_COMMUNICATION or HPI_ERROR_DSP_BOOTLOAD is
returned in wError.
*/
struct hpi_response_header {
	uint16_t wSize;
	uint8_t wType;		/* HPI_TYPE_RESPONSE  */
	uint8_t version;		/* response version */
	uint16_t wObject;		/* HPI_OBJ_* */
	uint16_t wFunction;		/* HPI_SUBSYS_xxx, HPI_ADAPTER_xxx */
	uint16_t wError;		/* HPI_ERROR_xxx */
	uint16_t wSpecificError;	/* Adapter specific error */
};

struct hpi_response {
#ifdef HPI_HEADER_AS_STRUCT
	struct hpi_response_header h;
#else
/* following fields must match HPI_RESPONSE_HEADER */
	uint16_t wSize;
	uint8_t wType;		/* HPI_TYPE_RESPONSE  */
	uint8_t version;		/* response version */
	uint16_t wObject;		/* HPI_OBJ_* */
	uint16_t wFunction;		/* HPI_SUBSYS_xxx, HPI_ADAPTER_xxx */
	uint16_t wError;		/* HPI_ERROR_xxx */
	uint16_t wSpecificError;	/* Adapter specific error */
#endif
	union {
		struct hpi_subsys_res s;
		union hpi_adapterx_res ax;
		struct hpi_stream_res d;
		struct hpi_mixer_res m;
		union hpi_mixerx_res mx;	/* extended mixer; */
		struct hpi_control_res c;	/* mixer control; */
		/* identical to hpi_control_res, but field naming is improved */
		union hpi_control_union_res cu;
		struct hpi_controlx_res cx;	/* extended mixer control; */
		struct hpi_nvmemory_res n;
		struct hpi_gpio_res l;	/* digital i/o */
		struct hpi_watchdog_res w;
		struct hpi_clock_res t;	/* dsp time */
		struct hpi_profile_res p;
		struct hpi_async_res as;
		uint8_t bytes[52];
	} u;
};

#define HPI_RESPONSE_SIZE_BY_OBJECT { \
	sizeof(struct hpi_response_header) ,/* Default, no object type 0 */ \
	sizeof(struct hpi_response_header) + sizeof(struct hpi_subsys_res),\
	sizeof(struct hpi_response_header) + sizeof(union  hpi_adapterx_res),\
	sizeof(struct hpi_response_header) + sizeof(struct hpi_stream_res),\
	sizeof(struct hpi_response_header) + sizeof(struct hpi_stream_res),\
	sizeof(struct hpi_response_header) + sizeof(struct hpi_mixer_res),\
	sizeof(struct hpi_response_header) , /* no node response */ \
	sizeof(struct hpi_response_header) + sizeof(struct hpi_control_res),\
	sizeof(struct hpi_response_header) + sizeof(struct hpi_nvmemory_res),\
	sizeof(struct hpi_response_header) + sizeof(struct hpi_gpio_res),\
	sizeof(struct hpi_response_header) + sizeof(struct hpi_watchdog_res),\
	sizeof(struct hpi_response_header) + sizeof(struct hpi_clock_res),\
	sizeof(struct hpi_response_header) + sizeof(struct hpi_profile_res),\
	sizeof(struct hpi_response_header) + sizeof(struct hpi_controlx_res),\
	sizeof(struct hpi_response_header) + sizeof(struct hpi_async_res) \
}

/*********************** version 1 message/response **************************/
#define HPINET_ETHERNET_DATA_SIZE (1500)
#define HPINET_IP_HDR_SIZE (20)
#define HPINET_IP_DATA_SIZE (HPINET_ETHERNET_DATA_SIZE - HPINET_IP_HDR_SIZE)
#define HPINET_UDP_HDR_SIZE (8)
#define HPINET_UDP_DATA_SIZE (HPINET_IP_DATA_SIZE - HPINET_UDP_HDR_SIZE)
#define HPINET_ASI_HDR_SIZE (2)
#define HPINET_ASI_DATA_SIZE (HPINET_UDP_DATA_SIZE - HPINET_ASI_HDR_SIZE)

#define HPI_MAX_PAYLOAD_SIZE (HPINET_ASI_DATA_SIZE - 2)


/* New style message/response, but still V0 compatible */
struct hpi_msg_adapter_get_info {
	struct hpi_message_header h;
};

struct hpi_res_adapter_get_info {
	struct hpi_response_header h; //v0
	struct hpi_adapter_res p;
};

/* padding is so these are same size as v0 hpi_message */
struct hpi_msg_adapter_query_flash {
	struct hpi_message_header h;
	uint32_t dwOffset;
	uint8_t pad_to_version0_size[
		sizeof(struct hpi_message) - /* V0 res */
		sizeof(struct hpi_message_header) -
		1 * sizeof(uint32_t)];
};

/* padding is so these are same size as v0 hpi_response */
struct hpi_res_adapter_query_flash {
	struct hpi_response_header h;
	uint32_t dwChecksum;
	uint32_t dwLength;
	uint32_t dwVersion;
	uint8_t pad_to_version0_size[
		sizeof(struct hpi_response) - /* V0 res */
		sizeof(struct hpi_response_header) -
		3 * sizeof(uint32_t)];
};

struct hpi_msg_adapter_start_flash{
	struct hpi_message_header h;
	uint32_t dwOffset;
	uint32_t dwLength;
	uint32_t dwKey;
	uint8_t pad_to_version0_size[
		sizeof(struct hpi_message) - /* V0 res */
		sizeof(struct hpi_message_header) -
		3 * sizeof(uint32_t)];
};

struct hpi_res_adapter_start_flash {
	struct hpi_response_header h;
	uint8_t pad_to_version0_size[
		sizeof(struct hpi_response) - /* V0 res */
		sizeof(struct hpi_response_header)];
};

struct  hpi_msg_adapter_program_flash_payload {
	uint32_t dwChecksum;
	uint16_t wSequence;
	uint16_t wLength;
	uint16_t wOffset; /**< offset from start of msg to data */
	uint16_t wUnused;
	/* ensure sizeof(header + payload) == sizeof(hpi_message_V0)
	  because old firmware expects data after message of this size */
	uint8_t pad_to_version0_size[
		sizeof(struct hpi_message) - /* V0 message */
		sizeof(struct hpi_message_header) -
		sizeof(uint32_t) -
		4 * sizeof(uint16_t) ];
};

struct  hpi_msg_adapter_program_flash{
	struct hpi_message_header h;
	struct  hpi_msg_adapter_program_flash_payload p;
	uint32_t data[256];
};

struct hpi_res_adapter_program_flash {
	struct hpi_response_header h;
	uint16_t wSequence;
	uint8_t pad_to_version0_size[
		sizeof(struct hpi_response) - /* V0 res */
		sizeof(struct hpi_response_header) -
		sizeof(uint16_t)];
};

struct hpi_msg_adapter_debug_read {
	struct hpi_message_header h;
	uint32_t dwDspAddress;
	uint32_t dwCountBytes;
};

struct hpi_res_adapter_debug_read {
	struct hpi_response_header h;
	uint8_t bytes[256];
};

#if 1
#define hpi_message_header_v1 hpi_message_header
#define hpi_response_header_v1 hpi_response_header
#else
/* V1 headers in Addition to v0 headers */
struct hpi_message_header_v1 {
	struct hpi_message_header h0;
/* struct {
} h1; */
};

struct hpi_response_header_v1 {
	struct hpi_response_header h0;
	struct {
		uint16_t wAdapterIndex;	/* the adapter index */
		uint16_t wObjIndex;		/* object index */
	} h1;
};
#endif

#ifndef HPI_OS_LINUX_KERNEL
// STRV HPI Packet
struct hpi_msg_strv {
	struct hpi_message_header h;
	struct hpi_entity strv;
};

struct hpi_res_strv {
	struct hpi_response_header h;
	struct hpi_entity strv;
};
#define MIN_STRV_PACKET_SIZE sizeof(struct hpi_res_strv)
#endif

struct hpi_msg_payload_v0 {
	struct hpi_message_header h;
	union {
		struct hpi_subsys_msg s;
		union hpi_adapterx_msg ax;
		struct hpi_stream_msg d;
		struct hpi_mixer_msg m;
		union hpi_mixerx_msg mx;
		struct hpi_control_msg c;
		struct hpi_control_union_msg cu;
		struct hpi_controlx_msg cx;
		struct hpi_nvmemory_msg n;
		struct hpi_gpio_msg l;
		struct hpi_watchdog_msg w;
		struct hpi_clock_msg t;
		struct hpi_profile_msg p;
		struct hpi_async_msg as;
	} u;
};

struct hpi_res_payload_v0 {
	struct hpi_response_header h;
	union {
		struct hpi_subsys_res s;
		union hpi_adapterx_res ax;
		struct hpi_stream_res d;
		struct hpi_mixer_res m;
		union hpi_mixerx_res mx;
		struct hpi_control_res c;
		union hpi_control_union_res cu;
		struct hpi_controlx_res cx;
		struct hpi_nvmemory_res n;
		struct hpi_gpio_res l;
		struct hpi_watchdog_res w;
		struct hpi_clock_res t;
		struct hpi_profile_res p;
		struct hpi_async_res as;
	} u;
};

union hpi_message_buffer_v1 {
	struct hpi_message m0; /* version 0 */
	struct hpi_message_header_v1 h;
	uint8_t buf[HPI_MAX_PAYLOAD_SIZE];
};

union hpi_response_buffer_v1 {
	struct hpi_response r0; /* version 0 */
	struct hpi_response_header_v1 h;
	uint8_t buf[HPI_MAX_PAYLOAD_SIZE];
};

compile_time_assert((sizeof(union hpi_message_buffer_v1) <= HPI_MAX_PAYLOAD_SIZE),
	message_buffer_ok);
compile_time_assert((sizeof(union hpi_response_buffer_v1) <= HPI_MAX_PAYLOAD_SIZE),
	response_buffer_ok);

/*////////////////////////////////////////////////////////////////////////// */
/* declarations for compact control calls  */
struct hpi_control_defn {
	uint8_t wType;
	uint8_t wChannels;
	uint8_t wSrcNodeType;
	uint8_t wSrcNodeIndex;
	uint8_t wDestNodeType;
	uint8_t wDestNodeIndex;
};

/*////////////////////////////////////////////////////////////////////////// */
/* declarations for control caching (internal to HPI<->DSP interaction)      */

/** indicates a cached uint16_t value is invalid. */
#define HPI_CACHE_INVALID_UINT16 0xFFFF
/** indicates a cached short value is invalid. */
#define HPI_CACHE_INVALID_SHORT -32768

/** A compact representation of (part of) a controls state.
Used for efficient transfer of the control state
between DSP and host or across a network
*/
struct hpi_control_cache_info {
	/** one of HPI_CONTROL_* */
	uint8_t ControlType;
	/** The total size of cached information in 32-bit words. */
	uint8_t nSizeIn32bitWords;
	/** The original index of the control on the DSP */
	uint16_t ControlIndex;
};

struct hpi_control_cache_vol {
	struct hpi_control_cache_info i;
	short anLog[2];
	unsigned short flags;
	char padding[2];
};

struct hpi_control_cache_meter {
	struct hpi_control_cache_info i;
	short anLogPeak[2];
	short anLogRMS[2];
};

struct hpi_control_cache_channelmode {
	struct hpi_control_cache_info i;
	uint16_t wMode;
	char temp_padding[6];
};

struct hpi_control_cache_mux {
	struct hpi_control_cache_info i;
	uint16_t wSourceNodeType;
	uint16_t wSourceNodeIndex;
	char temp_padding[4];
};

struct hpi_control_cache_level {
	struct hpi_control_cache_info i;
	short anLog[2];
	char temp_padding[4];
};

struct hpi_control_cache_tuner {
	struct hpi_control_cache_info i;
	uint32_t dwFreqInkHz;
	uint16_t wBand;
	short sLevelAvg;
};

struct hpi_control_cache_aes3rx {
	struct hpi_control_cache_info i;
	uint32_t dwErrorStatus;
	uint32_t dwFormat;
};

struct hpi_control_cache_aes3tx {
	struct hpi_control_cache_info i;
	uint32_t dwFormat;
	char temp_padding[4];
};

struct hpi_control_cache_tonedetector {
	struct hpi_control_cache_info i;
	uint16_t wState;
	char temp_padding[6];
};

struct hpi_control_cache_silencedetector {
	struct hpi_control_cache_info i;
	uint32_t dwState;
	char temp_padding[4];
};

struct hpi_control_cache_sampleclock {
	struct hpi_control_cache_info i;
	uint16_t wSource;
	uint16_t wSourceIndex;
	uint32_t dwSampleRate;
};

struct hpi_control_cache_microphone {
	struct hpi_control_cache_info i;
	uint16_t phantom_state;
	char temp_padding[6];
};

struct hpi_control_cache_generic {
	struct hpi_control_cache_info i;
	uint32_t dw1;
	uint32_t dw2;
};

struct hpi_control_cache_single {
	union {
		struct hpi_control_cache_info i;
		struct hpi_control_cache_vol vol;
		struct hpi_control_cache_meter meter;
		struct hpi_control_cache_channelmode mode;
		struct hpi_control_cache_mux mux;
		struct hpi_control_cache_level level;
		struct hpi_control_cache_tuner tuner;
		struct hpi_control_cache_aes3rx aes3rx;
		struct hpi_control_cache_aes3tx aes3tx;
		struct hpi_control_cache_tonedetector tone;
		struct hpi_control_cache_silencedetector silence;
		struct hpi_control_cache_sampleclock clk;
		struct hpi_control_cache_microphone microphone;
		struct hpi_control_cache_generic generic;
	} u;
};

struct hpi_control_cache_pad {
	struct hpi_control_cache_info i;
	uint32_t dwFieldValidFlags;
	uint8_t cChannel[8];
	uint8_t cArtist[40];
	uint8_t cTitle[40];
	uint8_t cComment[200];
	uint32_t dwPTY;
	uint32_t dwPI;
	uint32_t dwTrafficSupported;
	uint32_t dwTrafficAnouncement;
};

/* 2^N sized FIFO buffer (internal to HPI<->DSP interaction) */
struct hpi_fifo_buffer {
	uint32_t dwSize;
	uint32_t dwDSPIndex;
	uint32_t dwHostIndex;
};

#ifndef DISABLE_PRAGMA_PACK1
#pragma pack(pop)
#endif
/* SANITISE DELETE */
typedef struct hpi_control_msg HPI_CONTROL_MSG;
typedef struct hpi_controlx_msg HPI_CONTROLX_MSG;
typedef struct hpi_control_union_msg HPI_CONTROL_UNION_MSG;
typedef struct hpi_control_res HPI_CONTROL_RES;
typedef struct hpi_controlx_res HPI_CONTROLX_RES;
typedef union hpi_control_union_res HPI_CONTROL_UNION_RES;
typedef struct hpi_resource HPI_RESOURCE;
typedef struct hpi_msg_format HPI_MSG_FORMAT;
typedef union hpi_version HPI_VERSION;
typedef struct hpi_message HPI_MESSAGE;
typedef struct hpi_response HPI_RESPONSE;
typedef struct hpi_data_compat32 HPI_DATA_COMPAT32;
/* SANITISE DELETE END */

#ifndef HPI_ON_DSP
/* skip host side function declarations for DSP
   compile and documentation extraction */

char HPI_HandleObject(const hpi_handle_t dwHandle);

HPI_API_VOID (void) HPI_HandleToIndexes(
	const hpi_handle_t dwHandle,
	uint16_t *pwAdapterIndex,
	uint16_t *pwObjectIndex
);

HPI_API (hpi_handle_t) HPI_IndexesToHandle(
	const char cObject,
	const uint16_t wAdapterIndex,
	const uint16_t wObjectIndex
);

/*////////////////////////////////////////////////////////////////////////// */

/* main HPI entry point */
HPI_API_VOID (void) HPI_Message(
	HPI_MESSAGE *phm,
	HPI_RESPONSE *phr
);

#ifndef HPI_BUILD_KERNEL_MODE
/* UDP message */
void HPI_MessageUDP(
	HPI_MESSAGE *phm,
	HPI_RESPONSE *phr,
	const unsigned int nTimeout
);

uint16_t HPI_DriverOpen(void);
void HPI_DriverClose(void);
#endif

/* used in PnP OS/driver */
HPI_API (hpi_err_t) HPI_SubSysCreateAdapter(
	const hpi_hsubsys_t *phSubSys,
	const struct hpi_resource *pResource,
	uint16_t *pwAdapterIndex
);

HPI_API (hpi_err_t) HPI_SubSysDeleteAdapter(
	const hpi_hsubsys_t *phSubSys,
	uint16_t wAdapterIndex
);

HPI_API (hpi_err_t) HPI_OutStreamHostBufferGetInfo(
	const hpi_hsubsys_t *phSubSys,
	hpi_handle_t hOutStream,
	uint8_t ** ppBuffer,
	struct hpi_hostbuffer_status **ppStatus
);

HPI_API (hpi_err_t) HPI_InStreamHostBufferGetInfo(
	const hpi_hsubsys_t *phSubSys,
	hpi_handle_t hInStream,
	uint8_t ** ppBuffer,
	struct hpi_hostbuffer_status **ppStatus
);

HPI_API (hpi_err_t) HPI_AdapterRestart(
	uint16_t adapterIndex
);

/*
The following 3 functions were last declared in header files for
driver 3.10. HPI_ControlQuery() used to be the recommended way
of getting a volume range. Declared here for binary asihpi32.dll
compatibility.
*/
#ifndef HPI_OS_LINUX_KERNEL
HPI_API (hpi_err_t) HPI_ControlParamSet(
	const hpi_hsubsys_t *phSubSys,
	const hpi_handle_t hControl,
	const uint16_t wAttrib,
	const uint32_t dwParam1,
	const uint32_t dwParam2
);
HPI_API (hpi_err_t) HPI_ControlParamGet(
	const hpi_hsubsys_t *phSubSys,
	const hpi_handle_t hControl,
	const uint16_t wAttrib,
	uint32_t dwParam1,
	uint32_t dwParam2,
	uint32_t *pdwParam1,
	uint32_t *pdwParam2
);
HPI_API (hpi_err_t) HPI_ControlQuery(
	const hpi_hsubsys_t *phSubSys,	///< Pointer to HPI subsystem handle
	const hpi_handle_t hControl,	///< Control to query
	const uint16_t wAttrib,	///< An attribute of the control
	const uint32_t dwIndex,	///< Index for possible attribute values
	const uint32_t dwParam,	///< Supplementary parameter
	/** One of N possible settings for the control attribute,
	   specified by dwIndex=0..N-1, and possibly depending on the value
	   of the supplementary parameter) */
	uint32_t *pdwSetting
);
#endif

void HPI_FormatToMsg(
	struct hpi_msg_format *pMF,
	const struct hpi_format *pF
);
void HPI_StreamResponseToLegacy(
	struct hpi_stream_res *pSR
);

/*////////////////////////////////////////////////////////////////////////// */
/* declarations for individual HPI entry points */
HPI_HandlerFunc HPI_1000;
HPI_HandlerFunc HPI_6000;
HPI_HandlerFunc HPI_6205;
HPI_HandlerFunc HPI_COMMON;

#endif				/* ndef HPI_ON_DSP */

#ifdef __cplusplus
/* *INDENT-OFF* */
}
/* *INDENT-ON* */
#endif
#endif				/* _HPI_INTERNAL_H_ */

// $Log: hpi_internal.h,v $
// Revision 1.129  2011/02/03 14:26:11  as-age
// comment
//
// Revision 1.128  2010/12/14 01:35:25  as-ewb
// HPI_MessageUDP is not in kernel mode
//
// Revision 1.127  2010/12/06 23:42:44  as-ewb
// Comment update. Increase default network timeout to 500ms.
// Issue#3167
//
// Revision 1.126  2010/11/30 22:45:21  as-ewb
// changing flags field in hpi_control_cache_vol to ushort appears
// to avoid problems for asi5000 - byte accesses appear unsafe on
// its dsp!?
//
// Revision 1.125  2010/11/30 21:49:17  as-ewb
// DSP-only flag moved to dsp code
//
// Revision 1.124  2010/11/30 02:53:20  as-ewb
// Neither linux nor wdm kernel drivers define Volatile now,
// so remove it.
//
// Revision 1.123  2010/11/25 23:20:48  as-ewb
// Define a number of volume flags shared between host cache and dsp.
// Issue#3191
//
// Revision 1.122  2010/11/23 23:31:21  as-ewb
// Add HPI_VOLUME_GAIN_AND_FLAGS to help asi2400
// mute save/restore. Issue#2804
//
// Revision 1.121  2010/11/23 18:25:22  as-tfe
// Implemented stream wait functions.  Issue#2753
//
// Revision 1.120  2010/11/23 02:36:15  as-ewb
// Convert most conditional compilation define names to HPI_BUILD_x
//
// Revision 1.119  2010/11/19 21:14:46  as-age
// fill in gap in volume attribs.
//
// Revision 1.118  2010/11/19 20:46:37  as-age
// add mute capability to all volumes. Issue#2804
//
// Revision 1.117  2010/11/18 20:04:32  as-tfe
// Added IRQ callback HPI message.  Issue#2753
//
// Revision 1.116  2010/11/17 19:13:00  as-tfe
// Fix compile errors.
//
// Revision 1.115  2010/11/17 18:44:02  as-age
// IRQ tweaks
//
// Revision 1.114  2010/11/17 00:20:26  as-ewb
// Switch to using C99 fixed width types, and some lowercase
// hpi type names.  (Previous defs are still defined for compatibility)
//
// Revision 1.113  2010/11/11 15:43:24  as-age
// update mixer cache info parameters. Issue#3002
//
// Revision 1.112  2010/11/11 15:23:56  as-age
// add define for mixer_get_cache_info so that we can correctly get the cache
// size back from the 2416.
//
// Revision 1.111  2010/11/01 21:28:02  as-ewb
// tweak to help enum parser
//
// Revision 1.110  2010/10/21 23:48:38  as-ewb
// Get rid of handle unpacking macros hiding flow control.
//
// Revision 1.109  2010/10/21 03:37:52  as-ewb
// HPI Error code updates.
//
// Revision 1.108  2010/10/14 03:16:52  as-ewb
// Use URIs and handles to access subsys network settings.
// Issue#2728
//
// Revision 1.107  2010/10/12 01:41:47  as-ewb
// Use hpi_api_void for functions with void return type.
//
// Revision 1.106  2010/10/08 16:33:17  as-age
// tweaks to support msg/resp tunneling related getting 4.02 compander to show up
// in 4.04 (ie no firmware upgrade required). Issue#3025
//
// Revision 1.105  2010/10/06 02:03:17  as-ewb
// Exclude entity defs from linux kernel build.
//
// Revision 1.104  2010/10/05 04:04:20  as-ewb
// Remove hpi_network_ functions ids.
// Change function ids subsys_entity_x to subsys_option_x
//
// Revision 1.103  2010/09/30 01:02:20  as-ewb
// UnLock -> Unlock
//
// Revision 1.102  2010/09/28 03:38:58  as-ewb
// Define hpi v1 structs for debug read
//
// Revision 1.101  2010/09/24 04:42:45  as-ewb
// Silence detector count is not used by hpifunc.
//
// Revision 1.100  2010/09/21 21:27:09  as-ewb
// Get rid of entity value member, giving trouble to MS compiler too.
// Add macro to clarify value pointer calculation.
//
// Revision 1.99  2010/09/21 03:31:08  as-ewb
// minor unsigned char -> HW8 or uint8_t
//
// Revision 1.98  2010/09/21 01:55:05  as-ewb
// linux and wdm get pci info from os-specific structures.
// so remove unused fields from hpi_pci
//
// Revision 1.97  2010/09/20 22:29:11  as-ewb
// Restore the value field of hpi_entity  for host code, but not
// for DSP code.
//
// Revision 1.96  2010/09/17 02:48:12  as-ewb
// Simplify hpi_entity structure.  No need for separate header.
//
// Revision 1.95  2010/09/16 04:20:37  as-ewb
// Adding subsys entity and network functions for issue#2728
//
// Revision 1.94  2010/09/13 21:31:39  as-ewb
// Control cache structs can stand alone for each control type.
// (In this checkin, they still have exactly the same memory
// layout as before).
// Issue#3002
//
// Revision 1.93  2010/09/08 23:54:56  as-ewb
// Define default macros to extract PCI ids from hpi_pci.
//
// Revision 1.92  2010/09/07 02:36:47  as-ewb
// Minor tweaks to ease parsing.
//
// Revision 1.91  2010/09/01 21:54:27  as-ewb
// struct field names start with lowercase
//
// Revision 1.90  2010/08/27 04:36:55  as-ewb
// Avoid symbols, struct members etc that are a single word starting with uppercase.
// Eases code sanitisation for ALSA.
//
// Revision 1.89  2010/08/26 23:27:00  as-ewb
// Use enums to group related constants.
// Fix adapter restart keys for ASI2416 (mistakenly made
// not backward compatible in rev 1.84)
//
// Revision 1.88  2010/08/26 05:11:50  as-ewb
// Move HPI_ADAPTER_ILLEGAL to hpi_internal and
// rename it to HPI_ADAPTER_INDEX_INVALID
//
// Revision 1.87  2010/08/24 21:45:34  as-ewb
// Give control cache union elements more meaninful names.
// Issue#3002
//
// Revision 1.86  2010/08/20 22:17:49  as-ewb
// Make different tuner levels into separate attributes.
// Return and cache tuner level in a (signed) short, avoiding casts.
// Make special cache error values internal.
//
// Revision 1.85  2010/08/18 23:46:13  as-ewb
// Remove duplicate channelmode declarations.
// Tuner level kind enum should not be public.
// Add SNR level kind, for issue#2982 test.
//
// Revision 1.84  2010/08/12 02:58:16  as-ewb
// Get rid of overloaded use of hpi_adapter_[msg/res] members.
// Use hpi_adapterx exclusively.
//
// Revision 1.83  2010/08/12 00:06:21  as-ewb
// create an adapterx struct for asserts.
// fields mirror existing generic ones.
//
// Revision 1.82  2010/07/30 13:36:18  as-tfe
// Move and rename hpios files for WDM.  issue#2979
//
// Revision 1.81  2010/07/30 03:10:57  as-ewb
// Update profile parameter names, documentation,
// and struct element names to match actual implementation.
//
// Revision 1.80  2010/07/01 04:59:43  as-ewb
// Strip out lockedmem debug for alsa.  Could go in wdm header file?
//
// Revision 1.79  2010/06/28 23:28:58  as-ewb
// Churn the compander attributes ;)
//
// Revision 1.78  2010/06/28 22:58:02  as-ewb
// anLog cache elements are signed
//
// Revision 1.77  2010/06/28 15:21:39  as-age
// minimize compander attribute value "churn"
//
// Revision 1.76  2010/06/28 14:52:58  as-age
// split compander time constants across compressor and noise gate, issue#2867
//
// Revision 1.75  2010/06/25 05:18:35  as-ewb
// remove unused typedef
//
// Revision 1.74  2010/06/18 15:09:45  as-age
// use #define for extracting control from attribute
//
// Revision 1.73  2010/06/18 02:31:47  as-ewb
// Add individual compander attributes. Issue#2870
//
// Revision 1.72  2010/06/02 18:08:14  as-tfe
// Added ability to log file and line number for all locked memory allocation and free
// calls by defining HPI_LOG_LOCKED_MEM.  It also elevates the level of the
// locked memory log messages from DEBUG to WARNING.
//
// Revision 1.71  2010/05/14 04:00:13  as-ewb
// Define hpios_memalloczero
//
// Revision 1.70  2010/05/06 15:57:56  as-age
// add HDRadio tuner blend attribute - issue#2642
//
// Revision 1.69  2010/04/28 00:52:01  as-ewb
// Add mask and macro for module family extraction.
//
// Revision 1.68  2010/04/21 00:48:12  as-ewb
// Hide a few more macros not used by linux kernel build. Issue#1158
//
// Revision 1.67  2010/03/23 04:30:52  as-ewb
// Move hpi_bus enum to hpi_internal. Not used by any public function.
// Add 'NET' bus type.
// Issue#2728
//
// Revision 1.66  2010/03/19 14:50:05  as-age
// Add host side support for caching the phantom power attribute of a microphone control.
// Related to issue#2705.
//
// Revision 1.65  2010/02/26 03:56:01  as-ewb
// Make pointer parameters that are input only constistently "const"
//
// Revision 1.64  2010/02/09 20:14:34  as-age
// Add WAIT START inter-card sync functions. Not hooked up to anything yet.
//
// Revision 1.63  2010/02/02 02:27:25  as-ewb
// Remove unused attribute defines.
//
// Revision 1.62  2009/12/09 23:37:50  as-ewb
// Allow hpi_entity to compile with older TI compiler Issue#2479
//
// Revision 1.61  2009/11/24 20:52:57  as-ewb
// Internal HPI_Control functions are static in linux kernel build.
//
// Revision 1.60  2009/11/20 15:36:56  as-age
// remove  un-necessary HPI_API from hpifunc.c
//
// Revision 1.59  2009/11/20 15:03:02  as-age
// Move HPI_AdapterRestart() from hpifirmware.c to hpifunc.c. API remains the same.
// It is now declared in hpi_internal.h (ASI only fn.). Need to remove Doxygen comments.
//
// Revision 1.58  2009/11/06 19:35:16  as-tfe
// Change HPI_API macro to take return type as a parameter.
//
// Revision 1.57  2009/10/27 04:14:25  as-ewb
// match string delimiters in comment
//
// Revision 1.56  2009/10/08 03:39:48  as-ewb
// STR fields are unsigned
//
// Revision 1.55  2009/09/08 09:27:28  as-ewb
// Minor doxygen updates. Remove a little unused stuff.
//
// Revision 1.54  2009/07/16 23:52:35  as-ewb
// Rename padding field of control messages for use in ASI2xxx control save/restore
//
// Revision 1.53  2009/07/03 07:18:52  as-dxb
// Updated HPI_Entity_*() prototypes using the agreed parameter ordering
// Added doxygen docs to HPI_Entity_*()
// Updated Implementation of HPI_Entity_*() functions to be inline with the latest version of the entity API demo
// Removed HPI_ERROR_ENTITY_END_OF_DATA
// Added HPI_ERROR_ENTITY_ROLE_INVALID and HPI_ERR hpi_entity_check_role()
// Updated implementation of HPI_ERR hpi_entity_check_type()
// Fixed STR_ROLE_FIELD_MAX STR_TYPE_FIELD_MAX constants
// Removed HPI_Universal_Free() (always use HPI_Entity_Free())
//
// Revision 1.52  2009/07/02 00:13:39  as-dxb
// Redefined hpi_entity macros to inline functions and moved them into hpifunc.c
//
// Revision 1.51  2009/07/01 23:16:32  as-dxb
// Moved definition of the hpi_entity struct to hpi_internal.h. hpi.h now only contains an opaque (incomplete) type.
//
// Revision 1.50  2009/07/01 21:13:19  as-dxb
// Moved Entity related macros to hpi_internal.h
//
// Revision 1.49  2009/06/22 02:33:20  as-ewb
// Use wObjIndex for all stream and control messages. Clarify profile and nvmem .
//
// Revision 1.48  2009/06/18 23:34:59  as-ewb
// More hpi entity additions.
//
// Revision 1.47  2009/06/18 05:19:29  as-ewb
// Add strv defs
//
// Revision 1.46  2009/06/11 17:37:33  as-age
// More  HPI_InitMessageResponse() required padding.
//
// Revision 1.45  2009/06/11 17:11:29  as-age
// Since HPI_AdapterFirmwareGetInfo() calls HPI_InitMessageResponse() that in turn
// does memset(0) using sizeof(struct hpi_message), we need to pad struct hpi_msg_adapter_query_flash
// to be the same size as struct hpi_message.
//
// Revision 1.44  2009/06/06 01:06:39  as-age
// back out changes made in v1.43
//
// Revision 1.43  2009/06/05 13:09:04  as-age
// add generic v1 control msg/res structs
//
// Revision 1.42  2009/06/04 23:18:30  as-ewb
// merge from branch_hpimsg1 (See comments on 1.40.2.x versions)
// Move adapter IDs back to hpi.h per SGT request.
//
// Revision 1.41  2009/05/31 12:14:46  as-age
// add helper max() min() defines
//
// Revision 1.40.2.8  2009/06/03 23:20:59  as-ewb
// Move adapter IDs to hpi_internal. Add min,max macros
//
// Revision 1.40.2.7  2009/06/03 21:48:36  as-ewb
// formatting
//
// Revision 1.40.2.6  2009/05/25 23:48:23  as-ewb
// Make v0 header same as v1.  Allow experimentation with header as struct.
// Define padding based on struct sizes.
//
// Revision 1.40.2.5  2009/05/22 18:15:33  as-age
// see comment
//
// Revision 1.40.2.4  2009/05/22 08:13:56  as-ewb
// send enum item# for info query
//
// Revision 1.40.2.3  2009/05/22 04:21:47  as-ewb
// Flash prog block size is 256 words
//
// Revision 1.40.2.2  2009/05/22 04:12:53  as-ewb
// Add univ control.  Add v0 compatible messages for adapter flash ops.
//
// Revision 1.40.2.1  2009/05/21 23:32:37  as-ewb
// Unique control attributes. Backward compatible version in msg/rsp header
// Removed wDspIndex from message.
// Incompatible rearrangement of stream and control messages.
//
// Revision 1.40  2009/05/12 05:51:08  as-ewb
// add safe strncpy macros
//
// Revision 1.39  2009/04/28 03:12:01  as-ewb
// add 12 char string to control_union_res
//
// Revision 1.37  2009/04/27 23:37:34  as-ewb
// Universal control attributes
//
// Revision 1.36  2009/04/15 20:33:25  as-ewb
// add HPI_GENERIC_VALUE for asi2500
//
// Revision 1.35  2009/04/10 19:07:24  as-sgt
// make 3rd param of HPI_MessageUdp() definition be a const to match implementation
//
// Revision 1.34  2009/04/03 02:37:57  as-ewb
// replace hpios_c6000.h with local hpios.h
//
// Revision 1.33  2009/03/10 03:10:19  as-ewb
// Moved linux specific files into linux directories as hpios.[ch]
//
// Revision 1.32  2009/03/10 01:47:50  as-ewb
// Pull entire hpios.h verbatim into hpi_internal.h
//
// Revision 1.31  2009/02/10 01:38:50  as-ewb
// add missing subsys string. make compile_time_assert available everywhere
//
// Revision 1.30  2008/12/18 01:02:06  as-ewb
// rename to HPI_1000
//
// Revision 1.29  2008/12/17 19:06:43  as-age
// back out v1.28 control attribute and struct changes 'cause it breaks too much stuff.
//
// Revision 1.28  2008/12/15 21:45:23  as-age
// Since we are renumbering the control attributes, move the control index and control
// attribute to the top of all control msg structs. May as well have a single dose of pain !
//
// Revision 1.27  2008/12/15 18:56:44  as-age
// Add macro to extract the type of a control from it's attribute.
//
// Revision 1.26  2008/12/10 20:10:11  as-ewb
// Enable unique attributes for all controls
//
// Revision 1.25  2008/12/10 00:30:13  as-ewb
// clarify value of HIF_UNKNOWN constant
//
// Revision 1.24  2008/12/04 20:46:28  as-tfe
// Added hostbuffer get info messages and functions.
//
// Revision 1.23  2008/12/03 18:52:25  as-tfe
// Remove recent changes (revert to 1.19).
//
// Revision 1.22  2008/11/26 20:45:04  as-tfe
// Add dwOldBufferSize to stream response.
//
// Revision 1.21  2008/11/25 21:43:30  as-tfe
// Added user mode buffer access functions for in streams.
//
// Revision 1.20  2008/11/25 20:59:57  as-tfe
// Added user mode buffer access functions for out streams.  Functions in hpimsgx.c
// still unimplemented at this point.
//
// Revision 1.19  2008/09/02 13:11:20  as-age
// Add 50xx "lock" for local sample rate.
//
// Revision 1.18  2008/08/19 23:44:06  as-ewb
// remove funcs declared in hpimsginit.h
//
// Revision 1.17  2008/08/01 03:55:57  as-ewb
// remove remaining AES-18 defs
//
// Revision 1.16  2008/07/23 18:40:27  as-age
// HD-Radio -> HD Radio
//
// Revision 1.15  2008/07/18 05:44:44  as-ewb
// remove HPI_OS_DSP563xx cond compile
//
// Revision 1.14  2008/07/18 05:06:53  as-ewb
// remove STR_SIZE, no longer needed since hpi4000 removed
//
// Revision 1.13  2008/07/15 23:35:49  as-ewb
// a couple more internal structures
//
// Revision 1.12  2008/07/15 01:15:27  as-ewb
// merge from branc30711 - standalone hpi.h, hpi header includes use double quotes, remove win98
//
// Revision 1.11  2008/07/08 12:30:48  as-age
// Add HPI_GpioWriteStatus() that returns the current relay settings. GPIO response
// now contains 64 GPIO bits in a single response.
//
// Revision 1.10.2.1  2008/07/04 05:31:14  as-ewb
// switch to hpi_internal. Remove win16, dspcode array loader. hpi.h now standalone
//
// Revision 1.10  2008/07/02 09:23:25  as-ewb
// buffer commands moved here
//
// Revision 1.9  2008/07/01 21:49:14  as-ewb
// Issue#1518.  Control attributes now only in hpi_internal.h.
//
// Revision 1.8  2008/06/27 23:55:59  as-ewb
// fix sampleclock index enumeration.
// Convert autofade into a single attribute with profile and duration as values
//
// Revision 1.7  2008/06/27 04:38:41  as-ewb
// Correct compilation when NO_ MORE_PUBLIC_ATTRIBUTES defined
//
// Revision 1.6  2008/06/27 04:24:07  as-ewb
// Add sampleclock auto functions. Fix AUTO attribute index.
//
// Revision 1.5  2008/06/25 00:14:48  as-ewb
// Move most attributes into hpi_internal.h.
// Remove some deprecated defs
// If your app doesnt compile, copy the relevant bit back to hpi.h.
// Issue#1518
//
// Revision 1.4  2008/06/24 00:59:41  as-ewb
// Fix Log format
//
// Revision 1.3  2008/06/23 23:37:48  as-ewb
// add opening extern C
//
// Revision 1.2  2008/06/23 22:58:28  as-ewb
// move all from inside HPI_EXCLUDE_IMPLEMENTATION into hpi_internal.h
