/**
Copyright (C) 1997-2010 AudioScience, Inc. All rights reserved.

This software is provided 'as-is', without any express or implied warranty.
In no event will AudioScience Inc. be held liable for any damages arising
from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.
3. This copyright notice and list of conditions may not be altered or removed
   from any source distribution.

AudioScience, Inc. <support@audioscience.com>

( This license is GPL compatible see http://www.gnu.org/licenses/license-list.html#GPLCompatibleLicenses )
*/

#ifndef HPI_BUILD_NO_INTERRUPTS
struct hpi_adapter_obj;

/* a function that takes an adapter obj and returns an int */
typedef int adapter_int_func(struct hpi_adapter_obj *pao);
#endif

struct hpi_adapter_obj {
	struct hpi_pci pci;	// PCI info - bus#,dev#,address etc
	uint16_t wAdapterType;	// ASI6701 etc
	uint16_t wIndex;		//
	uint16_t wOpen;		// =1 when adapter open
	uint16_t wMixerOpen;

#ifdef HPI_LOCKING
	struct hpios_spinlock dspLock;
#endif

	uint16_t wDspCrashed;
	uint16_t wHasControlCache;
	void *priv;
#ifndef HPI_BUILD_NO_INTERRUPTS
	adapter_int_func *irq_query_and_clear;
#endif
};

struct hpi_control_cache {
	/** indicates whether the structures are initialized */
	uint16_t dwInit;
	uint16_t adap_idx;
	uint32_t dwControlCount;
	uint32_t dwCacheSizeInBytes;
	/** pointer to allocated memory of lookup pointers. */
	struct hpi_control_cache_info **pInfo;
	/** pointer to DSP's control cache. */
	uint8_t *pCache;
};

struct hpi_adapter_obj *HpiFindAdapter(uint16_t wAdapterIndex);

hpi_err_t HpiAddAdapter(struct hpi_adapter_obj *pao);

void HpiDeleteAdapter(struct hpi_adapter_obj *pao);

short HpiCheckControlCache(
	struct hpi_control_cache *pC,
	struct hpi_message *phm,
	struct hpi_response *phr
);
struct hpi_control_cache * HpiAllocControlCache(
	const uint32_t dwNumberOfControls,
	const uint32_t dwSizeInBytes,
	uint8_t *pDSPControlBuffer
);
void HpiFreeControlCache(struct hpi_control_cache *pCache);

void HpiCmn_ControlCache_SyncToMsg(
	struct hpi_control_cache *pC,
	struct hpi_message *phm,
	struct hpi_response *phr
);

hpi_err_t HpiValidateResponse(
	struct hpi_message *phm,
	struct hpi_response *phr
);

