/******************************************************************************
Copyright (C) 1997-2010 AudioScience, Inc. All rights reserved.

This software is provided 'as-is', without any express or implied warranty.
In no event will AudioScience Inc. be held liable for any damages arising
from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.
3. This copyright notice and list of conditions may not be altered or removed
   from any source distribution.

AudioScience, Inc. <support@audioscience.com>

( This license is GPL compatible see http://www.gnu.org/licenses/license-list.html#GPLCompatibleLicenses )

  Firmware downloader for ASI2400

 (C) Copyright AudioScience Inc. 2005
*******************************************************************************/
#include <string.h>
#include <stdlib.h>

#include "hpi_internal.h"
#include "hpimsginit.h"
#include "hpidspcd.h"
#include "hpidebug.h"
#include "hpinet.h"

#include "hpifirmware.h"

#define BOOTLOADER_OFFSET 0
#define FACTORY_OFFSET 0x08000
#define UPDATE_OFFSET  0x80000

struct fw_info {
	enum eHpiFirmwareId type;
	int offset;
	char name[12];
};

#define MAX_FILES 3
static struct fw_info fw[MAX_FILES] = {
	{HPI_FW_UPDATE, UPDATE_OFFSET, "Update"},
	{HPI_FW_FACTORY, FACTORY_OFFSET, "Factory"},
	{HPI_FW_BOOTLOADER, BOOTLOADER_OFFSET, "Bootloader"}
};

static hpi_err_t firmware_download(
	uint16_t wAdapterIndex,
	enum eHpiFirmwareId fwid,
	const char *fwpath,
	uint16_t nBootLoadFamily,
	uint16_t nBootLoadFamilyMask,
	progress_callback * pc,
	progress_callbackex * pcex,
	void *user_arg);

/*=================================================================*/
#ifdef _MSC_VER
static void delay(int millisecs)
{
	Sleep(millisecs);
}
#else
#include <time.h>

/*---------------------------------------------------------------------------*/
static void delay(long millisecs)
{
	struct timespec poll_interval = { 0, millisecs * 1000000l };
	nanosleep(&poll_interval, 0);
}
#endif

/* max 999 */
#define MSG_DELAY 10
/*=================================================================*/
/**

*/
static uint32_t calc_checksum(uint32_t checksum, int count, uint32_t *pData)
{
	int i;
	for (i = 0; i < count; i++)
		checksum ^= pData[i];

	return checksum;
}

/*=================================================================*/
/**

*/
static void HPI_CALLBACK null_progress_callback(char *state, uint32_t percent)
{
}

static void HPI_CALLBACK null_progressex_callback(enum eHpiFirmwareStatus status, uint32_t percent, void *user_arg)
{
}

/** number of words in code tailer */
#define TAILERSIZE 3
/*=================================================================*/
/**

*/
HPI_API (hpi_err_t) HPI_AdapterFirmwareDownload(
	uint16_t wAdapterIndex,
	enum eHpiFirmwareId fwid,
	const char *fwpath,
	uint16_t nBootLoadFamily,
	uint16_t nBootLoadFamilyMask,
	progress_callback * pc)
{
	return firmware_download(
		wAdapterIndex,
		fwid,
		fwpath,
		nBootLoadFamily,
		nBootLoadFamilyMask,
		pc,
		NULL,
		NULL);
}

HPI_API (hpi_err_t) HPI_AdapterFirmwareDownloadEx(
	uint16_t wAdapterIndex,
	enum eHpiFirmwareId fwid,
	const char *fwpath,
	uint16_t nBootLoadFamily,
	uint16_t nBootLoadFamilyMask,
	progress_callback * pc,
	progress_callbackex * pcex,
	void *user_arg)
{
	return firmware_download(
		wAdapterIndex,
		fwid,
		fwpath,
		nBootLoadFamily,
		nBootLoadFamilyMask,
		pc,
		pcex,
		user_arg);
}

static hpi_err_t firmware_download(
	uint16_t wAdapterIndex,
	enum eHpiFirmwareId fwid,
	const char *fwpath,
	uint16_t nBootLoadFamily,
	uint16_t nBootLoadFamilyMask,
	progress_callback * pc,
	progress_callbackex * pcex,
	void *user_arg)
{
	short error;
	struct dsp_code DspCode;
	uint32_t dwLength;
	uint32_t checksum;
	uint32_t total_checksum = 0;
	uint32_t dwStartOffset;
	uint32_t sequence = 0;
	uint32_t progress = 0;
	uint32_t p_inc;
	struct  hpi_msg_adapter_program_flash hm;
	struct  hpi_res_adapter_program_flash hr;

	delay(MSG_DELAY);

	if (!pc)
		pc = &null_progress_callback;
	if (!pcex)
		pcex = &null_progressex_callback;

	HpiOs_SetDspCodePath(fwpath);

	/* WARNING assumes knowledge of how the enums are ordered? */
	if ((fwid > HPI_FW_BOOTLOADER) || (fwid < HPI_FW_UPDATE))
		return HPI_ERROR_INVALID_OBJ_INDEX;

	dwStartOffset = fw[fwid - 1].offset;
	printf("Downloading %s image\n", fw[fwid - 1].name);

	if ((error = HpiDspCode_Open(nBootLoadFamily, &DspCode, NULL)) != 0)
		return (error);

	if (fwid == HPI_FW_BOOTLOADER) {
		/* Bootloader needs to be raw code image, so discard
		   the three header words.
		   Check length of first block is same as total file length-
		   overhead, to make sure there is only one block
		 */
		uint32_t dwAddress, dwType;

		HpiDspCode_ReadWord(&DspCode, &dwLength);
		HpiDspCode_ReadWord(&DspCode, &dwAddress);
		HpiDspCode_ReadWord(&DspCode, &dwType);

		if (dwLength != (DspCode.dwBlockLength - 4)) {
			return HPI_ERROR_DSP_FILE_FORMAT;
		}
		dwLength = DspCode.dwBlockLength - 4 + TAILERSIZE;
	} else {
		dwLength = DspCode.dwBlockLength + TAILERSIZE;
	}

	p_inc = 9000 * ARRAY_SIZE(hm.data)/ dwLength;
	pc("Erasing firmware (5-10 sec pause)\n", 0);
	pcex(HPI_FW_PROGSTATUS_ERASING, 0,user_arg);

	{
		struct hpi_msg_adapter_start_flash hmsf;
		struct hpi_res_adapter_start_flash hrsf;

		HPI_InitMessageResponse(
				(HPI_MESSAGE *)&hmsf,
				(HPI_RESPONSE *)&hrsf,
				HPI_OBJ_ADAPTER,
				HPI_ADAPTER_START_FLASH);

		hmsf.h.wAdapterIndex = wAdapterIndex;
		hmsf.dwKey = 0x0a51;
		hmsf.dwOffset = dwStartOffset;
		hmsf.dwLength = dwLength;

		hrsf.h.wError = 0;
		hrsf.h.wSize = sizeof(hrsf);

		HPI_MessageUDP((HPI_MESSAGE *)&hmsf, (HPI_RESPONSE *)&hrsf, 10000);	// wait up to 10 sec for flash erase
		error = hrsf.h.wError;
		delay(MSG_DELAY);
	}

	while ((DspCode.dwWordCount < DspCode.dwBlockLength) && !error) {
		uint16_t wLength;
		uint32_t *pdwCode;
		uint32_t dwRemain = DspCode.dwBlockLength - DspCode.dwWordCount;
		int retries;

		wLength = ARRAY_SIZE(hm.data);
		if (wLength > dwRemain)
			wLength = (unsigned short)dwRemain;

		error = HpiDspCode_ReadBlock(wLength, &DspCode, &pdwCode);
		if (error)
			break;

		checksum = calc_checksum(0, wLength, pdwCode);
		total_checksum ^= checksum;

		HPI_InitMessageResponse(
				(HPI_MESSAGE *)&hm,
				(HPI_RESPONSE *)&hr,
				HPI_OBJ_ADAPTER,
				HPI_ADAPTER_PROGRAM_FLASH);
		hm.h.wAdapterIndex = wAdapterIndex;
		hm.p.wSequence = (uint16_t)sequence;
		hm.p.dwChecksum = checksum;
		hm.p.wLength = wLength;
		hm.p.wOffset = offsetof(struct hpi_msg_adapter_program_flash, data);
		hm.h.wSize = hm.p.wOffset + wLength * sizeof(uint32_t);

		if (0) {
			int i;

			printf("Packet[%d]\n", sequence);
			printf("Checksum = 0x%08X, Length = 0x%X\n", checksum,
			       wLength);
			for (i = 0; i < wLength; i++) {
				printf("0x%08X ", pdwCode[i]);
				if ((i & 3) == 3)
					printf("\n");
			}
			printf("\n");
		}

		memcpy(&hm.data, pdwCode, wLength * sizeof(uint32_t));

		retries = 5;

		while (retries) {
			hr.h.wError = 0;
			hr.h.wSize = sizeof(hr);
			HPI_MessageUDP((HPI_MESSAGE *)&hm, (HPI_RESPONSE *)&hr, 2000);
			delay(MSG_DELAY);

			retries--;
			error = hr.h.wError;
			/* if ((sequence==3) && (retries == 4)) error = 1; // Test that repeated packet works  */
			if (!error)
				break;
			/*printf("Retry %d, seq %d\n",retries,sequence);*/
		}

		sequence++;
		progress += p_inc;
		pc("Programming %d\r", 10 + progress / 100);
		pcex(HPI_FW_PROGSTATUS_PROGRAMMING, 10 + progress / 100,user_arg);
	}

	if (!error) {
		HPI_InitMessageResponse(
				(HPI_MESSAGE *)&hm,
				(HPI_RESPONSE *)&hr,
				HPI_OBJ_ADAPTER,
				HPI_ADAPTER_PROGRAM_FLASH);
		hm.h.wAdapterIndex = wAdapterIndex;
		hm.p.wSequence = sequence++;

		hm.data[0] = total_checksum;
		hm.data[1] = DspCode.dwBlockLength;
		hm.data[2] = DspCode.dwVersion;

		hm.p.dwChecksum =
		    calc_checksum(0, TAILERSIZE, hm.data);
		hm.p.wLength = TAILERSIZE;
		hm.p.wOffset = offsetof(struct hpi_msg_adapter_program_flash, data);
		hm.h.wSize =
		    sizeof(hm.h) + sizeof(hm.p) + TAILERSIZE * sizeof(hm.data[0]);


		hr.h.wSize = sizeof(hr);

		HPI_MessageUDP((HPI_MESSAGE *)&hm, (HPI_RESPONSE *)&hr, 2000);
		delay(MSG_DELAY);

		error = hr.h.wError;
	}
	HpiDspCode_Close(&DspCode);
	pc("\nDone%d\n", 100);
	pcex(HPI_FW_PROGSTATUS_DONE, 100,user_arg);
	return error;
}

HPI_API (hpi_err_t) HPI_FileFirmwareGetInfo(const char *fwpath,
	uint16_t wAdapterFamily,
	uint16_t nAdapterFamilyMask,
	unsigned int *adapter,
	unsigned int *version,
	unsigned int *checksum,
	unsigned int *length)
{
	struct dsp_code DspCode;
	hpi_err_t error;

	HpiOs_SetDspCodePath(fwpath);

	error = HpiDspCode_Open(wAdapterFamily, &DspCode, NULL);
	if (error)
		return error;

	*version = DspCode.dwVersion;
	*checksum = DspCode.dwCrc;
	*length = DspCode.dwBlockLength;
	*adapter = DspCode.nAdapter;
	HpiDspCode_Close(&DspCode);
	return 0;
}

HPI_API (hpi_err_t) HPI_AdapterFirmwareGetInfo(uint16_t adapterIndex,
					   enum eHpiFirmwareId fwid,
					   unsigned int *version,
					   unsigned int *checksum,
					   unsigned int *length)
{
	struct hpi_msg_adapter_query_flash hm;
	struct hpi_res_adapter_query_flash hr;

	/* WARNING assumes knowledge of how the enums are ordered? */
	if ((fwid > HPI_FW_BOOTLOADER) || (fwid < HPI_FW_UPDATE))
		return HPI_ERROR_INVALID_OBJ_INDEX;

	HPI_InitMessageResponse(
			(HPI_MESSAGE *)&hm,
			(HPI_RESPONSE *)&hr,
			HPI_OBJ_ADAPTER,
			HPI_ADAPTER_QUERY_FLASH);
	hm.h.wAdapterIndex = adapterIndex;
	hm.dwOffset = fw[fwid - 1].offset;
	hr.h.wSize = sizeof(hr);

	HPI_MessageUDP((struct hpi_message *)&hm, (struct hpi_response *)&hr, 1000);

	*version = hr.dwVersion;
	*checksum = hr.dwChecksum;
	*length = hr.dwLength;

	return hr.h.wError;
}

